%% Scattered Acoustic Pressure From A Plane Wave Striking A Cylinder
% This script will calculate and plot the scattered pressure 
% from a plane wave striking an infinite rigid walled cylinder.
%
% Reference:
%
% * Miguel C. Junger and David Feit (1993) 
%   Sound structures, and their interaction",
%   Acoustical Society of America, through the 
%   American Institute of Physics, Woodbury, NY, USA, second edition,
%   ISBN 0262100347,
%
% Carl Howard 7 Feb 2013

%% Introduction
% The mathematical model described in Junger and Feit is an 
% acoustic plane wave that originates from the -x axis and propagates 
% to the +x axis. Hence in the polar / directivity plots, the plane wave
% originates from 180 degrees and travels radially inwards to the centre 
% and then outwards along the 0 degrees radial.
%
% The plane wave strikes a rigid walled infinitely long cylinder,
% of radius $a$ centred at the origin, at an angle normal to the axis of
% the cylinder.

%% Parameters of the cylinder
% Radius of the cylinder $a$
a_radius =  1.0;        % [m]

%% Parameters of the fluid
% speed of sound $c_0$
c_speed_sound   =   343.24;         % [m/s]

%%%
% Density of the fluid $\rho_0$
rho_density     =   1.2041;         % [kg/m^3]

%% Parameters of the analysis
% Wavelength $\lambda$
%
% Fig 8.1, p402 Morse and Ingard shows the scattered intensity at values of
%
% * $\lambda=2 \pi a$
% * $\lambda=\frac{2}{3} \pi a$
% * $\lambda=\frac{2}{5} \pi a$

%lambda_wavelength = 2*pi*a_radius;    % [m]
%lambda_wavelength = 2/3*pi*a_radius;    % [m]
lambda_wavelength = 2/5*pi*a_radius;    % [m]

%%%
% Other useful terms are
%
% Wavenumber $k=\omega/c_0=2\pi f /c = 2\pi / \lambda$
%
k_wavenum = 2 * pi / lambda_wavelength;

%%%
% Another parameter that is used frequently in the analysis is $ka$
ka  =   k_wavenum * a_radius;

%%%
% Define the angles where results will be calculate for 
% polar / directivity plots.
% In Morse and Ingard they use the symbol $\phi$ for angle.
%
% We'll define $\phi=-\pi \cdots \pi$ because the script to plot
% directivity prefers -180 deg to 180 deg, but it should work if
% this angle is changed to your preference.
phi_angle_junger = [-pi:0.1:pi];  % [radians]

%%%
% Radial distance where the pressure and intensity will be evaluated $r$
r_radius = 6.5;             % [m]

%% Parameters of the plane wave
% The amplitude of plane wave is $P_i$ [p322 Junger and Feit]
%
P_i = 1.0;          % [Pa] pressure amplitude of plane wave

%%% 
% Number of terms to approximate the cylindrical waves $n=0 \cdots \infty$,
% and is approximated as $n=0 \cdots$|n_max_index|.
%
% Change the value of |n_max_index| as needed. 
% Check to make sure that the plot for the incident plane wave 
% in the absence of the cylinder has a _constant_ sound pressure level 
% on the arc of constant radius. 
% The SPL should be the same throughout the acoustic domain, 
% for all angles and radii.
n_max_index = 50;               % change this value as required
n_index = [0:n_max_index];      % Must keep n_index(1)=0;


%% Calculate the incident plane wave in terms of cylindrical waves
% This is not used in the solution, but is calculated and plotted for
% verification purposes. 
%
% The plane wave can be written as [p322, Eq. (10.24) Junger and Feit]
%
% $$
%   p_p = P_i \sum_{n=0} \epsilon_{n} i^{n} J_{n}(kr)
%               \cos(n \phi) 
% $$
%
% Note that the plane wave is calculated at a constant radius |r_radius|
% over a range of angles |phi_angle|, so we are calculating the incident
% pressure on an *arc* ... This is _not_ the usual way a comparison 
% would be made, as normallly one would want to plot 
% the incident sound pressure level at |phi_angle=0| and 
% the radius would be varied.
%
% The value of $\epsilon_n$ is 
%
% $$ \epsilon_{0} = 1 $$ for $n=0$
%
% $$ \epsilon_{n} = 2 $$ for $n>0$  
%
epsilon_n       = 2*ones(size(n_index));    % for all values of n
epsilon_n(1)    = 1;                        % corresponding to n=0

p_plane_wave=zeros(size(phi_angle_junger));
J_n_kr = besselj(n_index,k_wavenum*r_radius);
for nn=1:length(phi_angle_junger)
    p_plane_wave(nn) = P_i *                                        ... 
                    (                                               ...
                    sum(    epsilon_n .*                            ...
                            1i.^(n_index) .*                        ...
                            J_n_kr .*                               ...
                            cos( n_index*phi_angle_junger(nn) )            ...
                          )                                         ...
                    );
end;

%% Plot the SPL of the incident plane wave
% Read the note from above -- this is the pressure on an arc at constant
% radius |r_radius| where the angle is varied.
%
% For a plane wave propagating in the absence of the cylinder, 
% the sound pressure level should be _constant_ throughout the domain, 
% for all angles and radii.
%
% If the plot does not show a (nearly) constant level (i.e. +/- 0.1 dB)
% then there is a problem with the calculations. Consider increasing the
% value of |n_max_index| and re-calculate this result.
%
% Calculate the sound pressure level as 
%
% $$
%   L_{p} = 20 \log_{10} 
%               \left[
%                   \frac{|p_{p}|}{\sqrt{2} \times 20 \times 10^{-6} }
%               \right]
% $$
p_plane_wave_dB_junger = 20*log10(  abs(p_plane_wave)/sqrt(2)/20e-6 ); 

% Plot the SPL for the incident plane wave in the absence of the cylinder
figure
set(gca,'fontsize',16);
p1h=plot(phi_angle_junger*180/pi,p_plane_wave_dB_junger);
title(['Incident Plane Wave Measured on an Arc of radius=',  ...
        num2str(r_radius),'m']);
xlabel('Angle \phi [degrees]');
ylabel('SPL [dB RMS re 20\muPa]');
set(p1h,'linewidth',3);


%% Calculate the scattered pressure
% The scattered pressure of the outgoing wave is 
% [Junger and Feit, p322, Eq. (10.27) ]
%
% $$ 
%   p_{s}(r,\phi) = -P_{i} \sum_{n=0}^{\infty} \epsilon_{n} i^{n}
%                   \frac{ J_{n}' (ka) H_{n}(kr) }{ H_{n}'(ka)} \,
%                   \cos( n \phi)
% $$
%
% where $H_{m}(kr)$ is the Hankel function
Hn_kr      =   besselh(n_index,1,(k_wavenum*r_radius));      

%%%
% The derivatives of the Bessel $J_n$, Neumann $N_n$, and Hankel $H_n$ 
% functions are
dJ_n_ka    =   besselj(n_index-1,(ka))-besselj(n_index+1,(ka));    
dN_n_ka    =   bessely(n_index-1,(ka))-bessely(n_index+1,(ka));
dH_n_ka    =   dJ_n_ka + 1i*dN_n_ka;

%%%
% Before evaluating the scattered pressure at each angle using 
% a |for| loop, a constant term can be pre-calculated 
%
% $$ 
%   \texttt{constant\_term} = \epsilon_{n} i^{n}
%                   \frac{ J_{n}'(ka) H_{n}(kr) }{ H_{n}'(ka)} 
% $$
constant_term = epsilon_n .*(1i).^(n_index) .* ((dJ_n_ka.*Hn_kr)./dH_n_ka);

%%%
% Use a |for| loop to calculate the scattered pressure at each angle of
% $\phi$.
%
p_scattered_junger = zeros(size(phi_angle_junger));          
for nn=1:length(phi_angle_junger)
    p_scattered_junger(nn) = -P_i * sum(                           ...
                            constant_term              .*   ...
                            cos(n_index*phi_angle_junger(nn))      ...
                          );                      
end;

%%%
% Calculate the Sound Pressure Level (SPL), which uses the 
% Root-Mean-Square (RMS) value of the scattered pressure 
% |p_scattered|, so it is necessary to divide the 
% peak pressure by $\sqrt{2}$.
p_scattered_dB_junger = 20*log10(  abs(p_scattered_junger)/sqrt(2)/20e-6 );


 
%% Plot the scattered pressure
figure
set(gca,'fontsize',16);
p2h=plot(phi_angle_junger*180/pi,p_scattered_dB_junger,'.');
set(p2h,'linewidth',3);
set(p2h,'markersize',16);
title(['Scattered Pressure at \lambda=',    ...
        num2str(lambda_wavelength),'m']);
xlabel('Angle \phi [degrees]');
ylabel('SPL [dB RMS re 20\muPa]')
l2h=legend('Junger and Feit',   ...
            'location','southeast');

