%% Calculate the Acoustic Power Out of a 2D hemisphere
% This function is intended for use with the 2D baffled piston analysis
% where an axi-symmetric model was created of a circular piston in an
% plane infinite baffle.
function [s]=power_freefield_2Dhemisphere(filename)
%%
% Calculates the acoustic power through a hemisphere
%
% This script uses data exported from Ansys to calculate the acoustic power
% by integrating the sound pressure level over a hemisphere. The hemisphere
% should have |FLUID129| elements overlayed on the surface, to simulate a
% free-field. 
%
% It is assumed that there are several files available:
%   filename.nodes.txt      - the exported node coordinates
%   filename.elems.txt      - the exported element data
%   filename.press.txt      - the exported pressure data
%   filename.freq.txt       - the list of frequencies
%
% The function returns information in a Matlab structure
% 
% * s.n  -> nodal information
% * s.e  -> elemental information
% * s.f  -> frequency information
% * s.p  -> mode shapes
% * s.m  -> modal mass
% * s.pl -> plots
% * s.id -> id string ('str')

%% Import the geometry of the hemisphere
% This section imports the node coordinates and elements of the FLUID129
% elements that were exported from Ansys.
% The node coordinates are used to calculate the effective nodal area and
% normals.
[n,e]=load_2Dhemi_geom(filename);

%%
% map node index to number
% The node numbers are stored in the vector n.num.
% These next lines create a vector called idx, that has a length 
% that is the maximum node number.
% Next a mapping is created to from the idx to the num vector.
% create a vector of zeros with a length that is the maximum node number
n.idx=zeros(max(n.num),1);

%%
% Loop through each entry for the node numbers and insert it into the idx
% vector at the correct location.
for ii=1:length(n.num);n.idx(n.num(ii))=ii;end

%%
% remove nodes not used in elements
elem_refnodes=union(e.i,e.j);
%elem_refnodes=union(elem_refnodes,e.k);
%elem_refnodes=union(elem_refnodes,e.l);


n.num=n.num(n.idx(elem_refnodes));
n.x=n.x(n.idx(elem_refnodes));
n.y=n.y(n.idx(elem_refnodes));
n.z=n.z(n.idx(elem_refnodes));

%%
% remap node index to number
n.idx=zeros(max(n.num),1);
for ii=1:length(n.num);n.idx(n.num(ii))=ii;end

%%
% calculate element areas and normals
% element points
e_pi=[n.x(n.idx(e.i)),n.y(n.idx(e.i)),n.z(n.idx(e.i))];
e_pj=[n.x(n.idx(e.j)),n.y(n.idx(e.j)),n.z(n.idx(e.j))];
%e_pk=[n.x(n.idx(e.k)),n.y(n.idx(e.k)),n.z(n.idx(e.k))];
%e_pl=[n.x(n.idx(e.l)),n.y(n.idx(e.l)),n.z(n.idx(e.l))];

%%
% element vectors
e_r1=0.5*(e_pj-e_pi);
% Define new vectors that are out of the XY plane - unit vectors along
% Z-axis
e_r2=repmat([0 0 1],size(e_r1,1),1);
e_r4=repmat([0 0 -1],size(e_r1,1),1);
% e_r2=0.5*(e_pk-e_pj);
% e_r3=0.5*(e_pl-e_pk);
% e_r4=0.5*(e_pi-e_pl);

%%
%element normals
e_ni=cross(e_r1,-e_r4);
e_nj=cross(e_r2,-e_r1);
% e_nk=cross(e_r3,-e_r2);
% e_nl=cross(e_r4,-e_r3);

%%
%element areas
e_ai=sqrt(sum(e_ni.^2,2));
e_aj=sqrt(sum(e_nj.^2,2));
% e_ak=sqrt(sum(e_nk.^2,2));
% e_al=sqrt(sum(e_nl.^2,2));

%%
% nodal area and norm
% must loop through elements, otherwise repeated nodes are not indexed
% properly
n.area=zeros(size(n.num));
n.norm=zeros(length(n.num),3);

% for ii=1:length(e.num)
%     elmidx=[n.idx(e.i(ii)),n.idx(e.j(ii)),n.idx(e.k(ii)),n.idx(e.l(ii))];
%     n.area(elmidx)=n.area(elmidx)+[e_ai(ii),e_aj(ii),e_ak(ii),e_al(ii)]';
%     n.norm(elmidx,:)=n.norm(elmidx,:)+  ...
%                         [e_ni(ii,:);e_nj(ii,:);e_nk(ii,:);e_nl(ii,:)];
% end;

for ii=1:length(e.num)
    elmidx=[n.idx(e.i(ii)),n.idx(e.j(ii))];
    n.area(elmidx)=n.area(elmidx)+[e_ai(ii),e_aj(ii)]';
    n.norm(elmidx,:)=n.norm(elmidx,:)+  ...
                        [e_ni(ii,:);e_nj(ii,:)];
end;

%n.norm=n.norm./(sqrt(sum(n.norm.^2,2))*[1 1 1]); % unity normalised
n.norm=n.norm./(sqrt(sum(n.norm.^2,2))*[1 1 0]); % unity normalised

e.idx=zeros(max(e.num),1);
for ii=1:length(e.num);
    e.idx(e.num(ii))=ii;
end;


%% Import the acoustic pressure data
% This section is used to import the acoustic pressure data from Ansys
pressure=load_2Dhemi_presure(filename);

%%
% Reduce the frequency vector from the repeated frequency listings
% (real and imaginary parts of the Ansys results) to a single
% entry
pressure.freq=reshape(pressure.freq,2,length(pressure.freq)/2).';
pressure.freq=pressure.freq(:,1);

%%
% Reduce the vector containing node numbers that are repeated for
% each analysis frequency, into a single vector that contains the node
% numbers.
% num = node number
pressure.num=pressure.num(1:length(pressure.num)/length(pressure.freq));

%%
% Check that the node numbers from the exported pressure results are
% identical to the node numbers found from the export of node coordinates.
% A more thorough way to check this is to go through node number by node
% number to ensure they are the same.
% The following test finds the absolute of the difference between the node
% numbers and then sums the entire vector. If there are any differences
% between the two vectors, the answer of the sum will be non-zero, and
% hence there is something wrong, so stop.
if(sum(abs(pressure.num-n.num))>0),
    error(['The node numbers from the pressure results do not match ', ...
            'exported node coordinates from NWRITE.']);
end;

%%
% map mode node index to number
% see the comments above where the node numbers mapped to the vector idx
pressure.idx=zeros(max(pressure.num),1);
for ii=1:length(pressure.num);
    pressure.idx(pressure.num(ii))=ii;
end;

pressure.num=pressure.num(pressure.idx(n.num));


%%
% remap mode node index to number
pressure.idx=zeros(max(pressure.num),1);
for ii=1:length(pressure.num);
    pressure.idx(pressure.num(ii))=ii;
end;

%%
% reshape the pressure vector into an array with dimensions:
%   (node vector) x (number of analysis frequencies)
pressure.pressure=reshape(pressure.pressure,[],length(pressure.freq));


%% Calculate the acoustic power radiating out of a hemisphere
%
% The transmitted power can be calculated by integrating the acoustic
% intensity over the surface of the hemisphere. 
% In discrete form this given by
%
% $$ W_{\rm{transmit}} = \sum_{n=1}^{N} I_{n} S_{n} $$
%
% $$ W_{\rm{transmit}} = \sum_{n=1}^{N} \frac{ p_{n}^{2} S_{n} }{\rho c} $$
%

%%
% Define the properties of the fluid
% Manually define the properties
%dens            = 1.208;    % [kg/m^3] density of fluid
%speed_sound     = 343;      % [m/s] speed of sound of fluid

%%
% or use the imported properties that have been read from the exported
% ANSYS results files.
speed_sound     = n.c;
dens            = n.dens;

%%
% Calculate the transmitted acoustic power through the hemisphere
% The power is calculated as
% 
% $$ L_w = \frac{P_n^2}{2 \rho c} \times A_n $$
% $$ {} = \frac{P_n^2}{2 \rho c} \times [l_n x_n 2 \pi ] $$
%
% where the effective nodal area is calculated using Pappus's Centroid
% Theorem: the effective nodal "length" is multiplied by the radius (the
% x-coordinate in this case) time 2 \pi. 
%
W_transmit  =   (abs(pressure.pressure.^2).' * (n.area.*n.x*2*pi)) / ...
                    (2*dens*speed_sound);



%%
% return a matlab structure containing the results
s.n=n;                  % node information
s.e=e;                  % element information
s.f=pressure.freq;      % frequency information
s.p=pressure;           % pressure results
s.power=W_transmit;      % acoustic power out of hemisphere
s.id='hemi';            % a text string 
%--------------------------------------------------
% End of function
%--------------------------------------------------



%% Matlab function to import the nodes and element data from ANSYS
% The node data should have been exported using NWRITE.
% The element data should have been exported using EWRITE using 
% the LONG format.
function [n,e]=load_2Dhemi_geom(filename);
%%
% Load geometry information
% Currently assumes that elements used are 2 node FLUID129. 

%%
% Load node and element data saved from ansys
[n.num,n.x,n.y,n.z] =  textread([filename,'.node.txt'],'%n%n%n%n');

%%
% Read the 14 columns from the .elem file and allocated into vectors
[e.i,e.j,e.mat,e.type,e.real,e.num] ...
    =  textread([filename,'.elem.txt'], ...
                    '%n%n%*n%*n%*n%*n%*n%*n%n%n%n%*n%*n%n');

% Load the properties of the fluid from the exported Ansys results
props=textread([filename,'.prop.txt'],'%s',-1);

%%
% Extract the speed of sound from the material property list.
% For Ansys v14.5, this is item #21 corresponds to SONC
% It might change with future versions of Ansys, 
% as it has changed from previous versions.
n.c=str2num(props{21});

%%
% Extract the density from the property list.
% For Ansys v14.5, this is item #15. Corresponds to property DENS.
% It might change with future versions of Ansys, 
% as it has changed from previous versions.
n.dens=str2num(props{15});
return;
%--------------------------------------------------
% End of function
%--------------------------------------------------


%% Matlab fuction to import the nodal pressures of the FLUID129 elements
% The FLUID129 element should be on the surface of a hemisphere.
function [p]=load_2Dhemi_presure(filename)
%%
% Load the acoustic pressure from the nodes of the FLUID129 elements.
% The acoustic pressure should have been exported from ANSYS using the
% command PRNSOL,PRES

[p.freq]=textread([filename,'.freq.txt'],'%*n%n%*n%*n%*n','headerlines',5);
[p.num,p_real]=textread([filename,'.p_re.txt'],'%n%n');
[p.num,p_imag]=textread([filename,'.p_im.txt'],'%n%n');

p.pressure=p_real + 1i*p_imag;
return;
%--------------------------------------------------
% End of function
%--------------------------------------------------


%% Matlab function to calculate nodal area
% The FLUID129 element should be on the surface of a hemisphere.
function [n]=load_2Dhemi_forces(filename)
%%
% Load the results from the static structural analysis where
% the FLUID129 elements were replaced with SHELL61 elements and the
% the nodal reaction forces were calculated.

[node,fx,fy,fz]=textread(['nodal_reaction_forces.txt'],'%n%n%n%n%*n',   ...
                            'headerlines',2);
%%
% As a pressure $P=1$ Pa was applied to the nodes, 
% the effective nodal area is the magnitude of the 
% reaction force. $P = |F| / A$, hence $A = |F| / P = |F| / 1$.
n.area = sqrt(fx.^2+fy.^2);
return;
%--------------------------------------------------
% End of function
%--------------------------------------------------
