%% Transmission Loss of an Expansion Chamber Silencer
% This script is used to calculate the transmission loss of a 
% single expansion chamber silencer
%%
% The goal of this script is to generate Fig 10.12, p386 from
% the book:
%
% * Beranek and Ver (1992),
%   "Noise and Vibration Control Engineering:
%   Principles and Applications",
%   John Wiley and Sons, New York, USA.
%   ISBN: 0-471-61751-2
%
% Keywords: 
% pipe ; duct ; 4 pole ;  four pole ; transmission line ;
% silencer; muffler; expansion chamber
%
% Carl Howard 24 April 2013

%% Define properties of the fluid
c = 343.24;          %[m/s] speed of sound
rho = 1.2041;        %[kg/m^3] density of air

%% Define the properties of the main duct
a = 0.05/2;        %[m] radius of main duct
S_duct = pi*(a^2);       %[m^2] area of main duct

%% Define the properties of the quarter wave tube
% Beranek and Ver (1992) Fig 10.12 shows the TL evaluated at 
% area ratios of 
% |N_ratio_areas| = 2, 4, 8, 16, 32, 64
% where 
%
% $$ 
%   N = \frac{S_2}{S_1} 
%   = \frac{S_{\textrm{expansion}}}{S_{\textrm{duct}}} 
% $$
%
% where 
%
% * $S_{\textrm{expansion}}$ is the cross sectional area of the 
% expansion chamber and
% * $S_{\textrm{duct}}$ is the cross sectional area of the main
% duct.
%
N_ratio_areas = 16;                  %[] ratio of areas
S_expansion = N_ratio_areas * S_duct;%[m^2] area of expansion chamber
L_expansion = 0.5;                   %[m] length of the expansion chamber

% S_expansion=pi*(0.2/2)^2;
% N_ratio_areas=S_expansion/S_duct;

%%%
% This is what would normally be used to define the area of the 
% expansion chamber.
%
%   a_expansion = 0.1;               %[m] radius of the expansion chamber
%   S_expansion = pi*a_expansion^2;    %[m^2] area of expansion chamber

%% Define the analysis frequency range
% Beranek and Ver (1992) Fig 10.12 has the x-axis range from 
% $(kL / \pi) = 0 \cdots 4$.
% Hence this determines the analysis frequency range.
k_max = 4 *pi / L_expansion;      %[] maximum wavenumber of analysis
f_max = k_max * c / (2*pi);     %[Hz] maximum analysis frequency
freq_spacing=1;               %[Hz] frequency increment
f = [1:freq_spacing:f_max];     %[Hz] Excitation frequency 
omega= 2*pi*f;                  %[rad/s] circular frequency
k = omega / c;                  % vector containing wavenumbers
kL = k*L_expansion;               % a parameter used numerous times

%% Calculate the transmission loss
% The transfer matrix for a resonator is given by
% [see Beranek and Ver (1992)], 
%
% $$ \mathbf{T} = 
%       \left[ 
%           \matrix{ T_{11} & T_{12}   \cr 
%                  T_{21} & T_{22} }  
%       \right]
%   = \left[ 
%       \matrix{
%       \cos (k L)  &   \frac{ j c_0 }{ S_{\textrm{expansion}} } \sin (k L)      \cr
%       \frac{j S_{\textrm{expansion}} }{c_0} \sin (k L)  &   \cos(k L)
%       }
%       \right] 
%   $$
%
T_11 = cos(kL);
T_12 = 1i*c/S_expansion * sin(kL);
T_21 = 1i*S_expansion/c * sin(kL);
T_22 = cos(kL);


%% Calculate the transmission loss
% The transmission loss is calculated from 
% Beranek and Ver (1992), Eq. (10.10) p374 as
%
% $$\textrm{TL} = 20 \log_{10} 
%   \left| 
%       \frac{T_{11}+ \frac{S}{c} T_{12} + \frac{c}{S} T_{21} + T_{22}}{2}
%   \right|     $$
%
TL_4pole = 20*log10( abs( T_11+S_duct/c*T_12 +c/S_duct*T_21 + T_22)/2);


%%%
% Jacobsen (2011) defines a single line equation for the 
% transmission loss of an expansion chamber as 
% [ Finn Jacobsen, (2011), September,
% "Propagation of sound waves in ducts", 
% Denmark Technical University, Note no 31260,
% <http://web-files.ait.dtu.dk/fjac/p_home_page/notes/Duct_acoustics.pdf> ]
%
% [see also Bies and Hansen (2009),
% "Engineering Noise Control", Eq. (9.99), p464]
%
% $$ \textrm{TL} = 10 \times \log_{10} 
%   \left[ 1 + \frac{1}{4} 
%       \left( \frac{S_1}{S_2} - \frac{S_2}{S_1} \right)^2
%               \sin^2 kL 
%   \right]
% $$
%
% where 
%
% * $S_1=$ |S_duct| is the cross sectional area of the 
%   upstream and downstream ducts.
% * $S_2=$ |S_expansion| is the cross sectional area of the 
%   expansion chamber, and is always $S_2 > S_1$
% * $L$ is the length of the expansion chamber.
%
TL_B_and_H = 10*log10( 1 + 0.25*(1/N_ratio_areas - N_ratio_areas)^2    ...
                    *( sin(kL) ).^2   );


%% Plot the transmission loss
x_axis=k*L_expansion/pi;          % normalised frequency
p1h=plot(x_axis,TL_4pole,'-',x_axis(1:20:end),TL_B_and_H(1:20:end),'r.');
grid
set(gca,'fontsize',16);
set(p1h,'linewidth',2);
set(p1h(2),'markersize',16);
l1=legend(  ['Beranek: N=',num2str(N_ratio_areas)],   ...
            ['B&H: N=',num2str(N_ratio_areas)]);uistack(p1h(2),'top');
axis([0 4 0 40]);
xlabel('Normalised Frequency kL/\pi');
ylabel('Transmission Loss [dB]');
title('TL of an Expansion Chamber Silencer');


%% Calculate the SPL along the duct
%
% The locations are
%
% * [1] outlet end
% * [2] exit of expansion chamber
% * {3] inlet of expansion chamber
% * [4] inlet end
%

L_upstream=0.5;         % [m] length of upstream duct before silencer
L_downstream=0.5;       % [m] length of downstream duct after silencer

%% Analysis frequency
freq=174;               % [Hz] Frequency of analysis

%%% 
% Wavelength $\lambda$
lambda = c / freq;

%%%
% Wave number $k$
k=(2*pi*freq) / c;

%% Define the acoustic excitation
% Define the particle velocity at source end of the duct
u_particle_vel_4=1;     %[m/s] driving end

%%%
% Mass volume velocity
V_4=rho*S_duct*u_particle_vel_4;     %[kg /s]
V_inlet = V_4;


%%%
% The transmission matrix for the outlet duct straight segment is
T_11    = cos(k * L_downstream);
T_12    = 1i*(c/S_duct) * sin(k * L_downstream);
T_21    = 1i*(S_duct/c) * sin(k * L_downstream);
T_22    = cos(k * L_downstream);
T_1     = [ T_11    T_12    ;   T_21    T_22];

%%% 
% The transmission matrix for the expansion chamber silencer is
T_11 = cos(k *L_expansion);
T_12 = 1i*c/S_expansion * sin(k *L_expansion);
T_21 = 1i*S_expansion/c * sin(k *L_expansion);
T_22 = cos(k *L_expansion);
T_2     = [ T_11    T_12    ;   T_21    T_22];

%%%
% The transmission matrix for the upstream inlet straight segment  is
T_11    = cos(k * L_upstream);
T_12    = 1i*(c/S_duct) * sin(k * L_upstream);
T_21    = 1i*(S_duct/c) * sin(k * L_upstream);
T_22    = cos(k * L_upstream);
T_3     = [ T_11    T_12    ;   T_21    T_22];

%%% 
% The total transmission matrix is
T_total = T_3 * T_2 * T_1;

%%%
% Decompose the total transmission matrix
T_11 = T_total(1,1);
T_12 = T_total(1,2);
T_21 = T_total(2,1);
T_22 = T_total(2,2);

%%%
% We know that the termination at the outlet is anechoic 
% (looking downstream) so it has an impedance 
%
% $$ Z = P_1 / u_1 = \rho_0 c_0 $$. 
%
% Hence the outlet pressure is 
%
% $$ P_1 = \rho_0 c_0 u_1 $$. 
%
% The mass volume velocity is 
%
% $$ V_1 = \rho S_{\textrm{duct}} u_1 $$
%
% So the outlet pressure is 
%
% $$ P_1 = \rho_0 c_0 \frac{V_1}{\rho_0 S_{\textrm{duct}}}
% = c_0 \frac{V_1}{S_{\textrm{duct}}} $$
%
% The same derivation applies at the inlet, as it has anechoic termination
% looking upstream. 
%
% $$ Z = P_4 / u_4 = \rho_0 c_0 $$
%
% Hence the inlet pressure is 
%
% $$ P_4 = \rho_0 c_0 u_4 $$
%
%
% The 4-pole matrices can be re-arranged to calculate the pressures at the
% inlet and outlet, as the inlet velocity and outlet impedances are known.
V_outlet = V_inlet / (T_11 + (S_duct/c) * T_12);
P_outlet = V_outlet * c / (S_duct);
P_inlet = T_11 * P_outlet + T_12 * V_outlet;




%% Calculate the pressure distribution along the length of the duct
% We know the pressure and velocity at the driving end of the duct 
% $P_4, V_4$ and at the outlet end $P_1, V_1$. We can define a 4-pole or
% transmission matrix for a small length of duct of length $\Delta z$ and
% incrementally calculate the pressure and velocity at the start of the
% duct. Hence we will be effectively traversing a "microphone" 
% from the outlet end of the duct to the driving end and calculating
% the pressure and velocity at each point.
%
% Note that we do not bother to calculate the response again 
% at $n=0$, $z=L$ because we already know that it is $[P_1 ; V_1]$.
%
% The sound pressure and velocity at microphone location $z_n$ is
%
% $$ \left[ \matrix{ P_{\textrm{mic}} \cr V_{\textrm{mic}} } \right] =
%           \left[\mathbf{T}_{\Delta z}\right]^{n} 
%       \left[ \matrix{ P_1 \cr V_1 } \right] $$
% 
% where $n = 1 \cdots$ |number_divs|. 
%
% Create a 4-pole matrix for a duct of length $\Delta z$
%
% $$ \mathbf{T}_{\Delta z} = \left[ 
%  \matrix{ \cos (k \Delta z) & j \frac{c}{S} \sin (k\Delta z)   \cr 
%   j \frac{S}{c} \sin (k\Delta z) & \cos (k\Delta z)   }  \right] $$
%
%
% Define a vector containing the positions of where pressures will be
% calculated (microphone location) in increments of $\Delta z$
number_divs = 1000;                 %[no units] number of duct segments

% Total length of the duct system
L_total_length = L_upstream + L_expansion + L_downstream;   %[m]

delta_z = L_total_length/number_divs;   %[m] length of each segment

%%%
% The transmission matrix for the outlet duct straight segment is
T_11    = cos(k * delta_z);
T_12    = 1i*(c /S_duct) * sin(k * delta_z);
T_21    = 1i*(S_duct / c) * sin(k * delta_z);
T_22    = cos(k * delta_z);
T_1_delta_z     = [ T_11    T_12    ;   T_21    T_22];

%%% 
% The transmission matrix for the expansion chamber silencer is
T_11 = cos(k * delta_z);
T_12 = 1i*c/S_expansion * sin(k * delta_z);
T_21 = 1i*S_expansion/c * sin(k * delta_z);
T_22 = cos(k * delta_z);
T_2_delta_z     = [ T_11    T_12    ;   T_21    T_22];

%%%
% The transmission matrix for the upstream inlet straight segment  is
T_11    = cos(k * delta_z);
T_12    = 1i*(c /S_duct) * sin(k * delta_z);
T_21    = 1i*(S_duct / c) * sin(k * delta_z);
T_22    = cos(k * delta_z);
T_3_delta_z     = [ T_11    T_12    ;   T_21    T_22];

%%%
% Calculate the response along the length of the duct.
%
% It is customary to define the inlet as $z=0$ and the outlet will be 
% $z=$ |L_total_length|. Hence the location of each duct segment will be 
%
% * [inlet:] $0 \leq z \leq$ |L_upstream|
% * [expansion:]  |L_upstream| $\leq z \leq$ |L_upstream + L_expansion|
% * [outlet:] |L_upstream + L_expansion| $\leq z \leq$ |L_total_length|
%
% Break the problem into 3 duct lengths and calculate the response over
% each of the segments then collate them.
%

%% Analyse the outlet duct
mic_loc_1=[0:delta_z:L_downstream];   %[m] microphone locations

%%%
% Define an empty matrix that contains the:
%
% * pressure (top row) and
% * velocity (bottom row) 
% along the length of the duct.
p_v_downstream=zeros(2,length(mic_loc_1));

%%%
% Insert the last column of the matrix with the pressure and velocity at
% the outlet end.
p_v_downstream(:,end) = [ P_outlet ; V_outlet];

%%%
% Calculate response along the duct
for index=length(mic_loc_1)-1:-1:1
        p_v_downstream(:,index)=T_1_delta_z*p_v_downstream(:,index+1);
end;


%% Analyse the expansion chamber
mic_loc_2=[0:delta_z:L_expansion];   %[m] microphone locations

%%%
% Define an empty matrix that contains the:
%
% * pressure (top row) and
% * velocity (bottom row) 
% along the length of the duct.
p_v_expansion=zeros(2,length(mic_loc_2));

%%%
% Insert the last column of the matrix with the pressure and velocity at
% the outlet end.
p_v_expansion(:,end) = p_v_downstream(:,1);

%%%
% Calculate response along the duct
for index=length(mic_loc_2)-1:-1:1
        p_v_expansion(:,index)=T_2_delta_z*p_v_expansion(:,index+1);
end;


%% Analyse the upstream inlet duct.
mic_loc_3=[0:delta_z:L_upstream];   %[m] microphone locations

%%%
% Define an empty matrix that contains the:
%
% * pressure (top row) and
% * velocity (bottom row) 
% along the length of the duct.
p_v_upstream=zeros(2,length(mic_loc_3));

%%%
% Insert the last column of the matrix with the pressure and velocity at
% the outlet end.
p_v_upstream(:,end) = p_v_expansion(:,1);

%%%
% Calculate response along the duct
for index=length(mic_loc_3)-1:-1:1
        p_v_upstream(:,index)=T_3_delta_z*p_v_upstream(:,index+1);
end;

%% Collate all the results
% Note that the last point in one segment should overlap with the first
% point in the next segment.
mic_loc = [ mic_loc_3,                          ...
            mic_loc_2+L_upstream,               ...
            mic_loc_3+L_upstream+L_expansion    ];

p_v =   [   p_v_upstream,               ...
            p_v_expansion,              ...
            p_v_downstream          ];

%% Plot the sound pressure level versus distance
SPL_axis    = 20*log10(abs(p_v(1,:)/20e-6/sqrt(2)));
SPL_inlet   = 20*log10(abs(P_inlet/20e-6/sqrt(2)));
SPL_outlet  = 20*log10(abs(P_outlet/20e-6/sqrt(2)));

figure
p2h=plot(mic_loc,SPL_axis,0,SPL_inlet,'x',L_total_length,SPL_outlet,'o');
grid
set(gca,'fontsize',16);
set(p2h,'linewidth',2);
set(p2h(2),'markersize',10);
set(p2h(3),'markersize',10);
xlabel('Axial Distance [m]')
ylabel('Sound Pressure Level [dB re 20 \muPa]');
title('SPL Along Duct with Expansion Chamber Silencer');
l2h=legend('Theory','Inlet','Outlet');

