%% Power from a piston attached to the end of a duct.
% This Matlab script can be used to plot the mechanical power 
%
%
%% Introduction
% This script will plot the power from a piston on the end of a 3m duct of
% diameter 0.1m where the far end of the duct radiates into a free-field
% with a planar baffle. In other words, the end of the duct can be
% considered as a flanged pipe.
%
% References for this theory are:
%
% * Lawrence E. Kinsler, Austin R. Frey, Alan B. Coppens, James V. Sanders,
%   (1999), "Fundamentals of Acoustics", 4th edition, Wiley, 
%   ISBN 0471847895.
% * Frank Fahy and Paolo Gardonio (2007),
%   "Sound and Structural Vibration: radiation transmission and response",
%   Second edition, Academic Press, ISBN 0123736331.
%
% Keywords: 
% pipe ; duct ; tube ; piston ; radiation ; flanged ; baffle
%
% Carl Howard 30 December 2012

%close all; clear all

%% Properties of the fluid
c_speed_sound=344;      % [m/s] Speed of sound in air at 20 degree c 
rho_density=1.21;       % [kg/m^3] Density of air 

%% Analysis frequency range
freq=[0:0.1:200];       % [Hz] Frequency of analysis

%%% 
% Wavelength $\lambda$
lambda = c_speed_sound* ones(size(freq)) ./ freq;

%%%
% Wave number $k=\omega / c = 2\pi f / c$
k_wave_no=(2*pi*freq)/c_speed_sound;

%% Properties of the duct
duct_diam=0.1;                      % [m] duct diameter
a_radius = duct_diam/2;             % [m] duct radius

%%%
% Cross sectional area of circular duct $S=\pi (a)^2$
S_duct_area=pi*a_radius^2;          % [m]

%%%
% Length of duct $L$
L_duct_length=3.0;                  % [m] Length of the duct


%%%
% Force on the piston $F_z$
F_z     =   1e-3;                   % [N]

%%
% A parameter that is used numerous times is $kL$
kL      = k_wave_no*L_duct_length;

%% Calculate the radiation from the pipe radiating into a plane baffle
% The radiation impedance from an open end of a pipe radiating 
% into a plane baffle is given by 
%
% [Eq. (10.2.10), p274 Kinsler et al. (1999)]
%
% $$ Z_{mL} = (\rho_0 c S) 
%                \left[
%                    \frac{1}{2}(ka)^2 + j \, \frac{8}{3 \pi} \, (ka)
%                \right]    $$
%
Z_mL_norm = 1/2*(k_wave_no * a_radius).^2 +    ...
            1i*(8/3/pi)*k_wave_no*a_radius;

%% Input mechanical impedance 
% The input mechanical impedance for the piston driving at the end 
% of the duct is 
%
% [Eq. (10.2.4), p273 Kinsler et al. (1999)]
%
% $$ Z_{m0} = (\rho_0 c S) \, 
%        \frac{ \frac{Z_{mL}}{(\rho_0 c S)} + j \tan (kL) }{
%               1 + j \frac{Z_{mL}}{(\rho_0 c S)} \tan (kL)}    $$
%
% Substitution of $Z_{mL}$ into the equation for $Z_{m0}$ gives
%
% $$ Z_{m0} = (\rho_0 c S) \, 
%        \frac{ \left[
%                   \frac{1}{2}(ka)^2 + j \, \frac{8}{3 \pi} \, (ka)
%                \right] + j \tan (kL) }{ 1 + j \left[
%                    \frac{1}{2}(ka)^2 + j \, \frac{8}{3 \pi} \, (ka)
%                \right] \tan (kL)}     $$
%
Z_m0 =     (rho_density*c_speed_sound*S_duct_area) * ...
            ( Z_mL_norm + 1i*tan(kL ) ) ./    ...
            ( 1+1i*Z_mL_norm .* tan(kL) );

%% Plot the imaginary mechanical impedance
figure
set(gca,'fontsize',16);
plot1h=plot(freq,imag(Z_m0),'-');
set(plot1h(:),'linewidth',2);
axis([0 200 -20 20]);
xlabel('Frequency [Hz]')
ylabel('Imaginary Impedance [Ns/m]')
title({'Imaginary Part of Mechanical Impedance',    ...
        'of Piston Attached to a Duct'})
leg1h=legend('Theory','location','southeast');
grid

%% Plot the real mechanical impedance
figure
set(gca,'fontsize',16);
plot2h=semilogy(freq,real(Z_m0));
set(plot2h(:),'linewidth',2);
xlabel('Frequency [Hz]')
ylabel('Real Impedance [Ns/m]')
title({'Real Part of Mechanical Impedance',    ...
        'of Piston Attached to a Duct'})
leg2h=legend('Theory Full Model: 1\timesRe(Z_{m0})','location','southeast');
grid

%% Power delivered into the piston
% The mechanical power done by the piston is given by
%
% [Eq. (2.4), p77 Fahy and Gardonio (2007)]
%
% $$ \textrm{Power} = \frac{1}{2} 
%           |\tilde{F}|^2 \, \textrm{Re}( \frac{1}{ Z_{m0} } ) $$
%
% Note there is a mistake in Kinsler et al. (1999) with the equation for
% power on p278.

power = F_z^2 /2 * real(ones(size(Z_m0))./Z_m0);

%% Plot the mechanical power
figure
set(gca,'fontsize',16);
plot3h=semilogy(freq,power);
set(plot3h(:),'linewidth',2);
xlabel('Frequency [Hz]')
ylabel('Power [Watts]')
title('Mechanical Power of Piston Attached to Duct')
leg3h=legend('Theory','location','southeast');
grid
