%% Sound Pressure Level Along a Duct Using the 4 Pole Method
% This Matlab script can be used to plot the pressure
% distribution along a cylindrical duct, where there is a 
% driving volume velocity at one end and the other end is rigid (closed).
%
%% Introduction
% The 4 pole (or transmission line) analysis method is used to calculate
% the pressure distribution along a duct.
%
% The theory and equations are adapted from: 
%
% * Leo L. Beranek and Istvan L. Ver (1992),
%   "Noise and Vibration Control Engineering: Principles and Applications",
%   John Wiley and Sons, New York, USA, ISBN: |0471617512|, 
%   page 377.
%
% Keywords: 
% pipe ; duct ; tube ; 4 pole ;  four pole ; transmission line ;
% pressure distribution
%
% Carl Howard 18 December 2012

%close all; clear all

%% Properties of the fluid
c_speed_sound=344;      % [m/s] Speed of sound in air at 20 degree c 
rho_density=1.21;       % [kg/m^3] Density of air 

%% Analysis frequency
freq=200;               % [Hz] Frequency of analysis

%%% 
% Wavelength $\lambda$
lambda = c_speed_sound / freq;

%%%
% Wave number $k$
k_wave_no=(2*pi*freq)/c_speed_sound;

%% Properties of the duct
duct_height=0.1;                    % [m] duct diameter

%%%
% Cross sectional area of circular duct $S$
S_duct_area=pi/4*duct_height^2;     % [m]

%%%
% Length of duct $L$
L_duct_len=3.0;                   % [m] Length of the duct

%%%
% Define a vector containing the positions of where pressures will be
% calculated (microphone location) in increments of $\Delta z$
number_divs = 1000;                 %[no units] number of duct segments
delta_z=L_duct_len/number_divs;   %[m] length of each segment
mic_loc=[0:delta_z:L_duct_len];   %[m] microphone locations

%%
% A parameter that is used numerous times is $kL$
kL      = k_wave_no*L_duct_len;

%% Define the acoustic excitation
% Define the particle velocity at source and rigid ends of the duct
u_particle_vel_1=0;     %[m/s] closed rigid end
u_particle_vel_2=1;     %[m/s] driving end

%%%
% Mass volume velocity
V_1=rho_density*S_duct_area*u_particle_vel_1;     %[kg /s]
V_2=rho_density*S_duct_area*u_particle_vel_2;     %[kg /s]

%% Calculate pressure at the driving end
% The impedance of a closed duct is
% 
% $$ Z_{\rm{tube}} = \frac{P_2}{V_2} $$
% $$ = - \frac{j c}{S} \cot (kL) $$
Z_tube = -1i*c_speed_sound/S_duct_area*cot(kL);

%%%
% Hence the pressure at the driving end is 
%
% $$ P_2 = Z_{\rm{tube}} \times V_2 $$
P_2 = Z_tube * V_2;

%% Calculate pressure at the closed end of the duct
% Define the 4-pole transmission matrix
% (see Beranek and Ver (1992), p377, Eq. (10.14) )
%
% $$ \left[ \matrix{ P_2 \cr \rho S u_2 } \right] =
%           \mathbf{T} 
%       \left[ \matrix{ P_1 \cr \rho S u_1 } \right] $$
%
% $$ \left[ \matrix{ P_2 \cr V_2 } \right] =
%           \mathbf{T} 
%       \left[ \matrix{ P_1 \cr V_1 } \right] $$
% 
% where (see Beranek and Ver (1992), p377, Eq. (10.15))
%
% $$ \mathbf{T} = \left[ 
%  \matrix{ \cos (kL) & j \frac{c}{S} \sin (kL)   \cr 
%   j \frac{S}{c} \sin (kL) & \cos (kL)   }  \right] $$
T_11    = cos(kL);
T_12    = 1i*(c_speed_sound/S_duct_area) * sin(kL);
T_21    = 1i*(S_duct_area/c_speed_sound) * sin(kL);
T_22    = cos(kL);
T       = [ T_11    T_12    ;   T_21    T_22];

%%%
% Calculate the inverse of the transmission matrix
%
% $$ \textbf{W} = \textbf{T}^{-1} $$
W=inv(T);

%%%
% Divide the matrix into elements
W_11 = W(1,1);
W_12 = W(1,2);
W_21 = W(2,1);
W_22 = W(2,2);

%%%
% Calculate pressure at the closed end of the duct $P_1$
P_1 = W_11 * P_2 + W_12 * V_2;
P_closed_end=P_1;

%%%
% Alternatively the 4-pole matrices can be re-arranged to calculate 
% $P_1$ and $P_2$, as the velocities $u_1$ and $u_2$ are known.
P_1_alt = (V_2 - T_22 * V_1) / T_21;
P_2_alt = T_11 * P_1_alt + T_12 * V_1;


%% Calculate the pressure distribution along the length of the duct
% We know the pressure and velocity at the driving end of the duct 
% $P_2, V_2$ and at the closed end $P_1, V_1$. We can define a 4-pole or
% transmission matrix for a small length of duct of length $\Delta z$ and
% incrementally calculate the pressure and velocity at the start of the
% duct. Hence we will be effectively traversing a "microphone" 
% from the closed end of the duct to the driving end and calculating
% the pressure and velocity at each point.
%
% There are |number_divs| small duct segments 
% (or divisions). 
% The location of the "microphone" is 
%
% $$ z_n = L - n \times \Delta z$$
%
% where $n = 0 \cdots$ |number_divs|. 
% Note that we do not bother to calculate the response again 
% at $n=0$, $z=L$ because we already know that it is $[P_1 ; V_1]$.
%
% The sound pressure and velocity at microphone location $z_n$ is
%
% $$ \left[ \matrix{ P_{\textrm{mic}} \cr V_{\textrm{mic}} } \right] =
%           \left[\mathbf{T}_{\Delta z}\right]^{n} 
%       \left[ \matrix{ P_1 \cr V_1 } \right] $$
% 
% where $n = 1 \cdots$ |number_divs|. 
%
% Create a 4-pole matrix for a duct of length $\Delta z$
%
% $$ \mathbf{T}_{\Delta z} = \left[ 
%  \matrix{ \cos (k \Delta z) & j \frac{c}{S} \sin (k\Delta z)   \cr 
%   j \frac{S}{c} \sin (k\Delta z) & \cos (k\Delta z)   }  \right] $$
T_11    = cos(k_wave_no*delta_z);
T_12    = 1i*(c_speed_sound/S_duct_area) * sin(k_wave_no*delta_z);
T_21    = 1i*(S_duct_area/c_speed_sound) * sin(k_wave_no*delta_z);
T_22    = cos(k_wave_no*delta_z);
T_dash       = [ T_11    T_12    ;   T_21    T_22];

%%%
% Define an empty matrix that contains the:
%
% * pressure (top row) and
% * velocity (bottom row) 
% along the length of the duct.
p_v=zeros(2,length(mic_loc));

%%%
% Insert the first column of the matrix with the pressure and velocity at
% the driving end.
p_v(:,end) = [ P_1 ; V_1];

%%%
% Calculate the response along the length of the duct
% Note that there are |number_divs+1| microphone positions
for index=length(mic_loc)-1:-1:1
    p_v(:,index)=T_dash*p_v(:,index+1);
end;

%% Calculate Sound Pressure Levels
% The sound pressure level at the microphone location is calculated as
%
% $$ \textrm{SPL}_{\textrm{mic}} = 
%     20 \log_{10} 
%           \left( 
%               \frac{|p_{\textrm{RMS}}|}{20 \times 10^{-6} } 
%           \right)     $$
%
% $$    \quad
%   = 20 \log_{10} 
%           \left( 
%               \frac{|p|}{\sqrt{2}} \times \frac{1}{20 \times 10^{-6} } 
%           \right)     
% $$

SPL_mic=20*log10(abs(p_v(1,:)/sqrt(2))/20e-6);

%%%
% For comparison, the SPL at the driving and closed end of the duct are
SPL_driving_end=20*log10(abs(P_2)/20e-6/sqrt(2));
SPL_closed_end=20*log10(abs(P_1)/20e-6/sqrt(2));

%% Plot the SPL along the length of the duct
figure
set(gca,'fontsize',16);
plot1h=plot(    mic_loc,SPL_mic,    ...
            0,SPL_driving_end,'x',    ...
            L_duct_len,SPL_closed_end,'o');
set(plot1h,'linewidth',2);
set(plot1h(2),'markersize',10);
xlabel('Mic Position Along Duct [m]'); 
ylabel('Sound Pressure Level [dB re 20uPa]');
title('Sound Pressure Level Along A Piston-Rigid Duct');
leg1h=legend( 'SPL along duct',       ...
        'SPL at driving end',   ...
        'SPL at rigid end',    ...
        'location','southwest');
axis([0 L_duct_len 100 150]);
box on
grid;

%% Plot the imaginary pressure along the length of the duct
figure
plot2h=plot(mic_loc,real(p_v(1,:)),     ...
            mic_loc,imag(p_v(1,:)),'--',     ...
            0,imag(P_2),'x',            ...
            L_duct_len,imag(P_1),'o');


set(gca,'fontsize',16);
set(plot2h,'linewidth',2);
set(plot2h(3),'markersize',10);
set(plot2h(4),'markersize',10);
xlabel('Mic Position Along Duct [m]'); 
ylabel('Pressure [Pa]');
title('Pressure Along A Piston-Rigid Duct');

leg2h=legend( 'Theory: Real pressure',       ...
        'Theory: Imag pressure',       ...
        'Im(P_2) at driving end',   ...
        'Im(P_1) at rigid end',    ...
        'location','EastOutside');
    
    axis([0 L_duct_len -500 500]);
box on
grid;



%% Plot the particle velocity along the duct
% The mass velocity is $V = \rho S u$ where $\rho$ is the density, 
% $S$ is the area of the duct, and $u$ is the particle velocity. 
% Hence the particle velocity is
%
% $$u_{\textrm{mic}} = \frac{V_{\textrm{mic}}}{\rho S} $$
%
u_mic = p_v(2,:)/rho_density/S_duct_area;
figure
plot3h=plot(    mic_loc,real(u_mic),    ...
            mic_loc,imag(u_mic),'--',    ...
            0,u_particle_vel_2,'x',    ...
            L_duct_len,u_particle_vel_1,'o');
set(gca,'fontsize',16);
set(plot3h,'linewidth',2);
set(plot3h(3),'markersize',10);
set(plot3h(4),'markersize',10);
xlabel('Mic Position Along Duct [m]'); 
ylabel('Particle Velocity [m/s]');
title('Particle Velocity Along A Piston-Rigid Duct');
leg3h=legend( 'Theory: Real particle vel.',       ...
            'Theory: Imag particle vel.',       ...
        'u_2 at driving end',   ...
        'u_1 at rigid end',    ...
        'location','EastOutside');
grid on;
