%% Impedance Tube with Abritrary Complex Impedance
% This file contains the analysis of an impedance tube
% which reads in data from Ansys.  The APDL script titled
% "code_ansys_impedance_3d.txt" should be in run in Ansys APDL prior to running
% this file.
%
% Note: If you click on File > Save and Publish to HTML you will have a
% nice record of this tutorial and its outputs.
%
% References:
% [1] A.F. Seybert and D.F. Ross. Experimental determination of acoustic properties
% using a two-microphone random-excitation technique. Journal of Acoustical
% Society of America, 61(5):13621370, 1977.
%
% [2] J.Y. Chung and D.A. Blaser. Transfer function method of measuring in-duct
% acoustic properties. i. theory. Journal of Acoustical Society of America, 68
% (3):907913, 1980.
%
% [3] G. Mller and M. Mser, editors. Handbook of Engineering Acoustics. Springer,
% 2013.
%
% Author: BSC
% Date: 6/6/2013
%
% Revision History
% Date      Change 
%
close all
clear all
clc


%% System parameters
% This cell is used to define all the system parameters

L = 1.0                                 % [m] Length of duct
X1 = 0.5                                % [m] Distance of Mic 1 to impedance
X2 = 0.6                                % [m] Distance of Mic 2 to impedance
c = 343.24                              % [m/s] Speed of sound in air
rho = 1.2041                            % [kg/m^3] Density of air
real_Z = rho*c                          % [N.s/m^3] Resistance of the material 
imag_Z = -rho*c                         % [N.s/m^3] Reactance of the material 



%% Load in Ansys Harmonic Analysis Results
% This cell is used to read in the results from the Ansys harmonic analysis
%ANSYS = 'Classic'           % Use classic
ANSYS = 'Workbench'         % Use workbench

switch ANSYS
    case('Classic')
        % Ansys Classic results
        % Microphone 1
        load ('../ansys/apdl/impd_3d_1.txt')
        % Microphone 2
        load ('../ansys/apdl/impd_3d_2.txt')

    case('Workbench')
        % Ansys Workbench results
        % Microphone 1
        impd_3d_1 = textread('../ansys/workbench/Impedance-3D-SF-IMPD_Mic1_Pres.txt','','headerlines',17,'delimiter',' ')
        % Microphone 2
        impd_3d_2 = textread('../ansys/workbench/Impedance-3D-SF-IMPD_Mic2_Pres.txt','','headerlines',17,'delimiter',' ')

    otherwise
        disp('Unknown method.')
end

f_ansys1 = impd_3d_1(:,1);             % Frequency vector
xfer_ansys1 = impd_3d_1(:,2) + j*impd_3d_1(:,3);   % Complex FRF
mag_ansys1 = abs(xfer_ansys1);            % Magnitude vector
phi_ansys1 = 180/pi*phase(xfer_ansys1);   % Phase vector

f_ansys2 = impd_3d_2(:,1);             % Frequency vector
xfer_ansys2 = impd_3d_2(:,2) + j*impd_3d_2(:,3);   % Complex FRF
mag_ansys2 = abs(xfer_ansys2);            % Magnitude vector
phi_ansys2 = 180/pi*phase(xfer_ansys2);   % Phase vector


%% Process Ansys Results
% This cell processes the Ansys results using the formulae defined below.
%
% The complex reflection coefficient is given by
%
% $$r = e^{2 j k X2}\frac{ H12-e^{-j k d} }{ e^{j k d} -H12} $$
% 
% The normal absorption coefficient is given by
%
% $$\alpha = 1-|r|^2 $$
%
% The surface impedance at the absorbing material is given by
%
% $$ Z = \frac{1+r}{1-r} $$
%

H12_ansys = xfer_ansys2./xfer_ansys1;     % Transfer function from Mic 1 to Mic 2
d = X2-X1;                          % Distance between mics
k = 2*pi*f_ansys1/c;                % Acoustic wavenumber


% Complex reflection coeff
r_ansys = exp(2*j*k*X2).*(( H12_ansys-exp(-j*k*d) )./( exp(j*k*d) -H12_ansys ) )
r_abs = abs(r_ansys)                % Magnitude of the reflection coefficient
alpha_ansys = 1-abs(r_ansys).^2     % Normal absorption coefficient

% Calculate the surface impedance at the absorbing material
Zs_ansys = (1+r_ansys)./(1-r_ansys)



%% Theoretical Parameters
% Directly calculate parameters

Zs = (real_Z + j *imag_Z)/rho/c     % Complex impedance
r_2 = (Zs-1)/(Zs+1)                 % Complex reflection coefficient
alpha_2  = 1 - abs(r_2)^2           % Sound absorption coefficient




%% Harmonic Response in the Duct
% Calculate the pressures at a number of locations in the duct to allow
% comparison with the Ansys results.
% 
% The pressure at a point in the duct (x metres from an impedance with 
% reflection coefficient r) is defined as the sum of forward and backwards
% travelling waves and is given by
%
% $$ p(x) = P_0(e^{jkx} + re^{-jkx}) $$
% 

% Define the range of frequencies to solve for
f = linspace(0.1,1000,1000);             % [Hz] Frequency range
omega = 2*pi*f;                         % [rad/s] Frequency range
rad2deg = 180/pi;                       % Radian to degree conversion
k = omega/c;                            % Wavenumber vector
s = j*omega;                            % Laplace variable

% Calculate transfer function between mics
p1 = exp(j*k*X1)+r_2*exp(-j*k*X1);      % Pressure at microphone 1     
p2 = exp(j*k*X2)+r_2*exp(-j*k*X2);      % Pressure at microphone 2

H12 = p1./p2;                           % Transfer function from p2 to p1




%% Plot the transfer function
figure(1)
set(gcf,'Name','Transfer function from mic 2 to mic 1')
subplot(2,1,1)                          % Bode magnitude
plot(f_ansys1, abs(H12_ansys),...       % Ansys magnitude
    '.k', 'MarkerSize',20)  
hold on                                 % Hold to overlay
plot(f, abs(H12), ...                   % Matlab magnitude
    'Color',[.5 .5 .5], 'LineWidth',2)  % Set to grey and width 2
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 100, 0, 5E-04])                % Set axis
xlabel('Frequency (Hz)')
ylabel('Magnitude')
grid


subplot(2,1,2)                          % Bode phase
plot(f_ansys1, angle(H12_ansys)*180/pi,...         % Ansys phase
    '.k', 'MarkerSize',20)
hold on
plot(f,angle(H12)*180/pi, ...                     % Matlab phase
    'Color',[.5 .5 .5], 'LineWidth',2)
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 100,-180,0])
xlabel('Frequency (Hz)')
ylabel('Phase, \phi (degrees)')
legend('ANSYS','MATLAB','Location','Best') 
grid

% Save the plot to a file
print -depsc 'transfer_function_3d_harm.eps'



%% Plot the impedance
figure(2)
set(gcf,'Name','Termination Impedance')


subplot(2,1,1)                          % Real
plot(f_ansys1, real(Zs_ansys),...       % Ansys real impendance
    '.k', 'MarkerSize',20)  
hold on                                 % Hold to overlay
plot(f, real(Zs)*ones(size(f)), ...                    % Matlab magnitude
    'Color',[.5 .5 .5], 'LineWidth',2)  % Set to grey and width 2
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
axis([0, 1000, 0.999, 1.001])                % Set axis
xlabel('Frequency (Hz)')
ylabel('Real')
grid


subplot(2,1,2)                          % Imaginary
plot(f_ansys1, imag(Zs_ansys),...       % Ansys imag impedance
    '.k', 'MarkerSize',20)
hold on
plot(f,imag(Zs)*ones(size(f)), ...      % Matlab imag impedance
    'Color',[.5 .5 .5], 'LineWidth',2)
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
axis([0, 1000,-1.001,-0.999])
xlabel('Frequency (Hz)')
ylabel('Imaginary')
legend('ANSYS','MATLAB','Location','Best') 
grid

% Save the plot to a file
print -depsc 'impedance_3d_harm.eps'



%% Tabulate scalar parameters
disp('Comparison of parameters at 100Hz')
disp(['Parameter        ','Matlab   ','Ansys    ','% Diff'])
disp(['Ref. Coeff.    ',num2str((r_2)),'  ',num2str((r_ansys(1))),'  '])
disp(['Mag Ref. Coeff.  ',num2str(abs(r_2)),'  ',num2str(abs(r_ansys(1))),'  ', ...
    num2str((abs(r_ansys(1))-abs(r_2))/abs(r_2)*100)])
disp(['Sound Ab. Coeff. ',num2str(alpha_2),'  ',num2str(alpha_ansys(1)),'  ', ...
    num2str((alpha_ansys(1)-alpha_2)/alpha_2*100)])
disp(['Real Imp. Ratio  ',num2str(real(Zs)),'  ',num2str(real(Zs_ansys(1))),'  ', ...
    num2str((real(Zs_ansys(1))-real(Zs))/real(Zs)*100)])
disp(['Imag Imp. Ratio  ',num2str(imag(Zs)),'  ',num2str(imag(Zs_ansys(1))),'  ', ...
    num2str((imag(Zs_ansys(1))-imag(Zs))/imag(Zs)*100)])

