%% Impedance Tube
% This file contains the analysis of an impedance tube
% Specifically:
%       % Frequency response
%
% Note: If you click on File > Save and Publish to HTML you will have a
% nice record of this tutorial and its outputs.
%
close all
clear all
clc


%% System parameters
L = 1.0                                 % [m] Length of duct
X1 = 0.5                                % [m] Distance of Mic 1 to impedance
X2 = 0.6                                % [m] Distance of Mic 2 to impedance
c = 344                                 % [m/s] Speed of sound in air
rho = 1.21                              % [kg/m^3] Density of air
mu = 0.5                                  % Boundary admittance



%% Load in Ansys Harmonic Analysis Results
% Microphone 1 - Pressure
load ('../ansys/apdl/impedance_1.txt')
f_ansys1 = impedance_1(:,1);             % Frequency vector
xfer_ansys1 = impedance_1(:,2) + j*impedance_1(:,3);   % Complex FRF
mag_ansys1 = abs(xfer_ansys1);            % Magnitude vector
phi_ansys1 = 180/pi*phase(xfer_ansys1);   % Phase vector 

% Microphone 2 - Pressure
load ('../ansys/apdl/impedance_2.txt')
f_ansys2 = impedance_2(:,1);             % Frequency vector
xfer_ansys2 = impedance_2(:,2) + j*impedance_2(:,3);   % Complex FRF
mag_ansys2 = abs(xfer_ansys2);            % Magnitude vector
phi_ansys2 = 180/pi*phase(xfer_ansys2);   % Phase vector 

% Microphone 3 - Pressure
load ('../ansys/apdl/impedance_3.txt')
f_ansys3 = impedance_3(:,1);             % Frequency vector
xfer_ansys3 = impedance_3(:,2) + j*impedance_3(:,3);   % Complex FRF
mag_ansys3 = abs(xfer_ansys3);            % Magnitude vector
phi_ansys3 = 180/pi*phase(xfer_ansys3);   % Phase vector 

% Microphone 3 - Velocity
load ('../ansys/apdl/impedance_4.txt')
f_ansys4 = impedance_4(:,1);             % Frequency vector
xfer_ansys4 = impedance_4(:,2) + j*impedance_4(:,3);   % Complex FRF
%xfer_ansys4 = impedance_4(:,2).*exp(j*impedance_4(:,3)/180*pi);   % Complex FRF
mag_ansys4 = abs(xfer_ansys4);            % Magnitude vector
phi_ansys4 = 180/pi*phase(xfer_ansys4);   % Phase vector 


%% Process Ansys Results
% First the two-microphone method
H12_ansys = xfer_ansys2./xfer_ansys1;     % Transfer function from Mic 1 to Mic 2
d = X2-X1;                          % Distance between mics
k = 2*pi*f_ansys1/c;                % Acoustic wavenumber
% Complex reflection coeff
r_ansys = exp(2*j*k*X2).*(( H12_ansys-exp(-j*k*d) )./( exp(j*k*d) -H12_ansys ) )
abs(r_ansys)
alpha_ansys = 1-abs(r_ansys).^2     % Normal absorption coefficient

% Calculate the surface impedance at the absorbing material
Zs_ansys = (1+r_ansys)./(1-r_ansys)

% Directly estimate the impedance ratio using the pressure and velocity at
% the surface
Zs_ansys2 = (xfer_ansys3./xfer_ansys4)/rho/c    % Normalised impedance
r_ansys2 = (Zs_ansys2-1)./(Zs_ansys2+1) % Reflection coefficient
alpha_ansys2 = 1-abs(r_ansys2).^2       % Normal absorption coefficient



%% Theory - Directly calculate known parameters
r_2 = (1/mu-1)/(1/mu+1)             % Complex reflection coefficient
alpha_2  = 1 - abs(r_2)^2           % Sound absorption coefficient
% Calculate the surface impedance at the absorbing material
Zs = (1+r_2)./(1-r_2)



%% HARMONIC RESPONSE
% Define the range of frequencies to solve for
f = linspace(0.1,1000,1000);             % [Hz] Frequency range
omega = 2*pi*f;                         % [rad/s] Frequency range
rad2deg = 180/pi;                       % Radian to degree conversion
k = omega/c;                            % Wavenumber vector
s = j*omega;                            % Laplace variable

% Calculate transfer function between mics
p0 = 1/rho./omega;                      % Pressure amplitude induced by flow
% Define the freq resp func, Eqn `(\ref{eq: single degree of freedom FRF})`
p1 = exp(j*k*X1)+r_2*exp(-j*k*X1);      % Pressure at microphone 1     
p2 = exp(j*k*X2)+r_2*exp(-j*k*X2);      % Pressure at microphone 2

H12 = p1./p2;                           % Transfer function from p2 to p1




%% Plot the transfer functions
figure(1)
set(gcf,'Name','Transfer function from mic 2 to mic 1')
subplot(2,1,1)                          % Bode magnitude
plot(f_ansys1, abs(H12_ansys),...         % Ansys magnitude
    '.k', 'MarkerSize',20)  
hold on                                 % Hold to overlay
plot(f, abs(H12), ...                    % Matlab magnitude
    'Color',[.5 .5 .5], 'LineWidth',2)  % Set to grey and width 2
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 100, 0, 5E-04])                % Set axis
xlabel('Frequency (Hz)')
ylabel('Magnitude')
grid


subplot(2,1,2)                          % Bode phase
plot(f_ansys1, angle(H12_ansys)*180/pi,...         % Ansys phase
    '.k', 'MarkerSize',20)
hold on
plot(f,angle(H12)*180/pi, ...                     % Matlab phase
    'Color',[.5 .5 .5], 'LineWidth',2)
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 100,-180,0])
xlabel('Frequency (Hz)')
ylabel('Phase, \phi (degrees)')
legend('ANSYS','MATLAB') 
grid

print -depsc 'impedance_harm.eps'





%% Plot the impedance
figure(2)
set(gcf,'Name','Termination Impedance')


subplot(2,1,1)                          % Real
plot(f_ansys1, real(Zs_ansys),...       % Ansys real impendance
    '.k', 'MarkerSize',20)  
hold on                                 % Hold to overlay
plot(f, real(Zs), ...                    % Matlab magnitude
    'Color',[.5 .5 .5], 'LineWidth',2)  % Set to grey and width 2
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 1000, 0.999, 1.001])                % Set axis
xlabel('Frequency (Hz)')
ylabel('Real')
grid


subplot(2,1,2)                          % Imaginary
plot(f_ansys1, imag(Zs_ansys),...       % Ansys imag impedance
    '.k', 'MarkerSize',20)
hold on
plot(f,imag(Zs), ...      % Matlab imag impedance
    'Color',[.5 .5 .5], 'LineWidth',2)
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 1000,-1.001,-0.999])
xlabel('Frequency (Hz)')
ylabel('Imaginary')
legend('ANSYS','MATLAB','Location','Best') 
grid

% Save the plot to a file
print -depsc 'impedance_harm2.eps'



%% Plot the absorption coefficient
figure(3)
set(gcf,'Name','Absorption Coefficient')
plot(f_ansys1, alpha_ansys,...       % Ansys real impendance
    '.k', 'MarkerSize',20)  
hold on                                 % Hold to overlay
plot(f, alpha_2, ...                    % Matlab magnitude
    'Color',[.5 .5 .5], 'LineWidth',2)  % Set to grey and width 2
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 1000, 0.999, 1.001])                % Set axis
xlabel('Frequency (Hz)')
ylabel('Absorption Coefficient')
grid
legend('ANSYS','MATLAB','Location','NorthWest') 

% Save the plot to a file
print -depsc 'impedance_harm3.eps'



%% Tabulate scalar parameters
disp(['Comparison of parameters at ',num2str(f_ansys1(1)),'Hz'])
disp(['Parameter        ',' Matlab  ','Ansys 2-mc','      % Diff ', ...
    '  Ansys p/v','      % Diff'])
Table_of_answers = [r_2, abs(r_ansys(1)), (abs(r_ansys(1))-r_2)/r_2*100 , ...
    abs(r_ansys2(1)), (abs(r_ansys2(1))-r_2)/r_2*100; ...
    alpha_2,alpha_ansys(1),(alpha_ansys(1)-alpha_2)/alpha_2*100, ...
    alpha_ansys2(1),(alpha_ansys2(1)-alpha_2)/alpha_2*100; ...
    real(Zs),real(Zs_ansys(1)),(real(Zs_ansys(1))-real(Zs))/real(Zs)*100, ...
    real(Zs_ansys2(1)),(real(Zs_ansys2(1))-real(Zs))/real(Zs)*100];
Text_Labels = ['Mag Ref. Coeff.  ';'Sound Ab. Coeff. ';'Real Imped. Ratio'];
disp([Text_Labels,num2str(Table_of_answers)])

