%% Plane Wave Viscous Losses
% This file contains the analysis of the attenuation of a plane wave 
% due to viscous losses.  
% 
% Calculations are based on theory as well as imported results from both
% Ansys Classic and Ansys Workbench.
%
% Note: If you click on File > Save and Publish to HTML you will have a
% nice record of this tutorial and its outputs.
%
% References:
% Alan D. Pierce. Acoustics: an introduction to its physical principles and
% applications. Acoustical Society of America, Woodbury, New York, USA,
% 1994.

% Author: BSC
% Date: 6/6/2013
%
% Revision History
% Date      Change 
% 

close all
clear all
clc


%% System parameters
%  Define the important parameters used in the analysis.

% Properties of the air
c_0 = 343.24                    % [m/s] Speed of sound in air
rho_0 = 1.2041                  % [kg/m^3] Density of air
%c_0 = 344                      % [m/s] Speed of sound in air
%rho_0 = 1.21                   % [kg/m^3] Density of air
mu_0 = 1.84E-5                  % [Pa.s] Dynamic viscosity of air
nu_0 = mu_0/rho_0               % [m^2/s] Kinematic viscosity of air

K = 0.0257                      % [W/m/K] Thermal Conductivity
c_p = 1012                      % [J/kg/K]Specific Heat Capacity at Constant Pressure
%
% $$ \gamma=\frac{c_{p}}{c_{v}} $$
%
gamma_0 = 1.4                   % Ratio of Specific Heats	
c_v = c_p/gamma_0               % [J/kg/K] Specific Heat Capacity at Constant Volume (per unit mass)

% Analysis options
f_0 = 100E3                     % [Hz] Frequency
omega_0 = 2*pi*f_0              % [rad/s] Angular frequency
lambda = c_0/f_0                % [m] Wavelength at chosen frequency



%% Theoretical Analysis
% The following cell has been used to calculate the expected attenuation
% based on classical theory.
%
% The attenuation is approximated by [Pierce] 
%
% $$ \alpha_{cl} = \frac{2 \omega^2 \nu}{3 c_0^3} $$
%
alpha_cl_viscous = (2*omega_0^2*nu_0)/(3*c_0^3)
alpha_cl_thermal = (omega_0^2*nu_0)/(2*c_0^3)*(gamma_0-1)*K/(c_p*mu_0)
alpha_cl = alpha_cl_viscous + alpha_cl_thermal

% The attenuation in dB/m is given by
Atten = -20*log10(exp(-alpha_cl))
8.6859* alpha_cl

% More correctly, it is better to calculate the complex wavenumber given by
% first defining
delta_cl = nu_0/2 * (4/3 + (gamma_0-1)*K/(c_p*mu_0))
k_0 = omega_0/c_0/sqrt(1+j*2*omega_0*delta_cl/c_0^2)

% The attenuation constant is given by
alpha_cl_2 = - imag(k_0)

% The attenuation in dB/m is given by
Atten = -20*log10(exp(-alpha_cl_2))
8.6859*alpha_cl_2

%% Create a structure containing the theoretical parameters to facilitate checking against Ansys
disp('Theory Parameters and Results')
theory.rho_0 = rho_0;   % Density of air
theory.c_0 = c_0;       % Speed of sound
theory.mu_0 = mu_0;     % Dynamic viscosity of air
theory.kappa_0 = K;     % Thermal conductivity of air
theory.c_p = c_p;       % Specific Heat Capacity at Constant Pressure
theory.c_v = c_v;		% Specific Heat Capacity at Constant Volume (per unit mass)
theory.Atten = Atten      % Attenuation in dB/m


%% Read in Ansys APDL results
% Check if APDL results are available
if exist('../ansys/apdl/visco_thermal_scalars.txt', 'file')
    % File exists.
    disp('ANSYS APDL Parameters and Results')
    load ../ansys/apdl/visco_thermal_scalars.txt
    apdl.rho_0 = visco_thermal_scalars(1);      % Density of air
    apdl.c_0 = visco_thermal_scalars(2);        % Speed of sound
    apdl.mu_0 = visco_thermal_scalars(3);       % Dynamic viscosity of air
    apdl.kappa_0 = visco_thermal_scalars(4);	% Thermal conductivity of air
    apdl.c_p = visco_thermal_scalars(5);        % Specific Heat Capacity at Constant Pressure
    apdl.c_v = visco_thermal_scalars(6);		% Specific Heat Capacity at Constant Volume (per unit mass)
    apdl.MIN_SPL = visco_thermal_scalars(7);	% Min SPL in dB
    apdl.MAX_SPL = visco_thermal_scalars(8);	% Max SPL
    apdl.Atten = visco_thermal_scalars(9)	% Attenuation in dB/m
else
    % File does not exist.
    warningMessage = sprintf(['Warning: file does not exist:\n ', ...
        '../ansys/apdl/visco_thermal_scalars.txt\n\n', ...
        'Run the ANSYS APDL script:\n',...
        'code_ansys_visco_thermal.txt']);
    uiwait(msgbox(warningMessage));
end


