%% Sound Pressure Level Inside a Rigid Walled Cavity
% This script is used to calculate the sound pressure at a point within a 
% rectangular cavity.
%%
% The script is used to verify the results
% predicted in ANSYS and the MATLAB scripts using a modal super-position
% (or often called modal summation) method.
%
% The reference for the theory is from 
%
% * Beranek and Ver (1992),
%   "Noise and Vibration Control Engineering:
%   Principles and Applications",
%   John Wiley and Sons, New York, USA.
%   ISBN: 0-471-61751-2
%   See page 151 onwards
%
% Another suitable reference is
%
% * P.A. Nelson and S.J. Elliot (1993),
%   "Active control of sound",
%   Academic Press
%   ISBN-10: 0125154267
%   ISBN-13: 978-0125154260
%   See Eq. (10.3.14), p316
%
% Keywords: 
% Rigid walled ; cavity; box ; resonance frequencies; modal summation;
% modal superposition
%
% Carl Howard 15 May 2013
% Modified by Ben Cazzolato to include spectral damping.


%% Define the system parameters
% properties of the fluid (air)
fluid.speedsound    = 343;      %[m/s] Speed of sound (air)
fluid.density       = 1.21;     %[kg/m^3] density of fluid (air)

%%%
% Dimensions of the cavity
cavity.length_x = 0.5;		%[m] length
cavity.length_y = 0.3;		%[m] width
cavity.length_z = 1.1;		%[m] height

%%%
% Volume of cavity
cavity.volume= abs(cavity.length_x .* cavity.length_y .* cavity.length_z);

%%%
% This is the number of modes that will be used in the modal summation
number_of_modes = 40; 

%%%
% This is the number of modes that will initially calculated along each
% axis, and then the resonance frequencies will be sorted and only
% |number_of_modes| will be retained for the analysis. As there are 3 axes,
% the number of resonance frequencies that will be calculated are
% |max_index_axes|^3.
max_index_axes = 40; 

%%%
% Vector of the analysis frequency range in Hz
freq=[1:0.1:500]';

%%%
% Volume velocity of source
%
% In Ansys we can apply an acoustic mass source at a point 
% with the APDL command 
%
% |BF,node,JS,masssource| 
%
% The |masssource| is defined as |masssource|= $\rho_0 Q $ kg/s, 
% where $\rho_0$ is the density, and $Q$ is the equivalent acoustic 
% volume velocity of the source.
%
% Hence, the volume velocity that is defined for the load excitation 
% in this Matlab model is calculated as
%
% $$ Q = \texttt{masssource} \, / \rho_0 $$
%
source.vol_velocity = 1 / fluid.density;     %[] volume velocity of source

%%%
% Location of the source
source.loc_x = 0.15;
source.loc_y = 0.12;
source.loc_z = 0.0;

%%%
% Evaluate the pressure at this (microphone) location 
%   (0.300, 0.105, 0.715).
result.loc_x = 0.3;
result.loc_y = 0.105;
result.loc_z = 0.715;

%% Check the definitions for dimensions are within range
% Check that the locations of the source and receiver are within the
% dimensions of the cavity
if abs(result.loc_x) > cavity.length_x
   error('result.loc_x is outside the dimensions of the cavity');    
end;
if abs(result.loc_y) > cavity.length_y
   error('result.loc_y is outside the dimensions of the cavity');
end;
if abs(result.loc_z) > cavity.length_z
   error('result.loc_z is outside the dimensions of the cavity');
end;

if abs(source.loc_x) > cavity.length_x
   error('source.loc_x is outside the dimensions of the cavity');    
end;
if abs(source.loc_y) > cavity.length_y
   error('source.loc_y is outside the dimensions of the cavity');
end;
if abs(source.loc_z) > cavity.length_z
   error('source.loc_z is outside the dimensions of the cavity');
end;

%% Calculate the Resonance Frequencies
% The resonance frequencies of the rigid walled cavity will be calculated
% for modal indicies $n=1 \cdots$ |max_index_axes|. As there are 3 axes
% (x,y,z), the number of resonance frequencies that are calculated is
% |max_index_axes|^3.
%
% The resonance frequencies of a rigid walled cavity are given by 
% [Beranek and Ver (1992) Eq. 6.13]
%
% $$
% f_{n}=\frac{c_0}{2}
%   \sqrt{
%   \left[\frac{n_{x}}{L_{x}}\right]^{2} 
%   + \left[\frac{n_{y}}{L_{y}}\right]^{2} 
%   + \left[\frac{n_{z}}{L_{z}}\right]^{2} 
%   } \quad \textrm{(Hz)}
% $$
% 
% where $c_0$ is the speed of sound, 
%   $n_x$ is the modal index along the $x$ axis, 
%   $L_x$ is the length of the cavity along the $x$ axis, 
%   and similarly for $n_y, n_z$ and $L_y, L_z$.

index=1;
cavity.res_freqs=zeros(max_index_axes^3,4);
for ii=0:max_index_axes,
    for jj=0:max_index_axes
        for kk=0:max_index_axes
            cavity.res_freqs(index,1) = fluid.speedsound/2 *       ...
                sqrt(   (ii/cavity.length_x).^2 +       ...
                        (jj/cavity.length_y).^2 +       ...
                        (kk/cavity.length_z).^2 );
            cavity.res_freqs(index,2) = ii;
            cavity.res_freqs(index,3) = jj;
            cavity.res_freqs(index,4) = kk;
            index = index + 1;
        end;
    end;
end;

%%%
% Sort the list of resonance frequencies into increasing values
cavity.res_freqs=sortrows(cavity.res_freqs,1);

%%%
% Only retain the first |number_of_modes| of resonance frequencies
cavity.res_freqs=cavity.res_freqs(1:number_of_modes,:);


%% Calculate the Modal Volume
% the modal volume for a rectangular box is calculated using 
% Beranek and Ver (1992) Eq 6.29 
% 
% $$ V_m = \textrm{volume} 
%           \times \epsilon_{nx} \times \epsilon_{ny} \times \epsilon_{nz} $$
%
% where $n_x, n_y, n_z$ are the modal indices along the $x,y,z$ axes, 
% respectively, and the value of $\epsilon$ is
%
% $\epsilon_n = 1$ if $n=0$, or 
%
% $\epsilon_n=0.5$ if $n>=1$
%
cavity.modalvolume=[];
cavity.modalvolume = 0.5 .* ones(number_of_modes,4);

% fix the 1st index == ii, X
index=[];
index=find(cavity.res_freqs(:,2)==0);
cavity.modalvolume(index,2)=1;
%fix the 2nd index == jj, Y
index=[];
index=find(cavity.res_freqs(:,3)==0);
cavity.modalvolume(index,3)=1;
%fix the 3rd index == kk, Z
index=[];
index=find(cavity.res_freqs(:,4)==0);
cavity.modalvolume(index,4)=1;

cavity.modalvolume(:,1) =   cavity.volume .*  ...
                            cavity.modalvolume(:,2) .*  ...
                            cavity.modalvolume(:,3) .*  ...
                            cavity.modalvolume(:,4);

%% Calculate the vectors of mode shapes                       
% The mode shapes have the function:
% see page 152, table 6.2 in Beranek and Ver (1992)
%
% $$ 
% \psi (x,y,z) =    \cos(n_x \pi x / L_x) \, 
%                   \cos(n_y \pi y / L_y) \, 
%                   \cos(n_z \pi z / L_z)
% $$
%
% Calculate the mode shape vector associated with the acoustic source
source.psi_Q_0=[];
source.psi_Q_0(:,1) =   ...
    cos( cavity.res_freqs(:,2).* pi .* source.loc_x./cavity.length_x).*...
    cos( cavity.res_freqs(:,3).* pi .* source.loc_y./cavity.length_y).*...
    cos( cavity.res_freqs(:,4).* pi .* source.loc_z./cavity.length_z);
 
%%%
% Calculate the mode shape vector associated with the receiver
result.psi_r=[];
result.psi_r(:,1) = ...
    cos( cavity.res_freqs(:,2) .* pi .* result.loc_x./cavity.length_x).*...
    cos( cavity.res_freqs(:,3) .* pi .* result.loc_y./cavity.length_y).*...
    cos( cavity.res_freqs(:,4) .* pi .* result.loc_z./cavity.length_z);


%% Calculate the pressure at the microphone by modal summation
pressure=[];
omega = 2 .* pi .* freq;

% Pre-calculate the term that does not alter with frequency
% and move it outside of the FOR loop.
term1 =  result.psi_r .* source.psi_Q_0 ./  ...
            cavity.modalvolume(:,1) ;
        
for ff=1:length(freq)
    p_temp=zeros(number_of_modes,1);
    p_temp = term1     ...
            ./ (omega(ff).^2 - (2.*pi.*cavity.res_freqs(:,1)).^2 );
    p_temp = sum(p_temp);
    pressure(ff,1) = fluid.density .* fluid.speedsound.^2 .* (-1i) .*   ...
                    omega(ff) .* source.vol_velocity .* p_temp;
end;



%% Compare the resonance frequencies calculated using Ansys and Matlab
% This section will plot a graph showing the resonance frequencies
% calculated using this Matlab script compared with the results from the
% Ansys Workbench analysis, and the Ansys Mechanical APDL analysis where
% the modal super-position technique was used.
%
% Load the resonance frequencies calculated from the MAPDL analysis
load('../ansys/apdl/ansys_resfreqs.txt');

%%%
% Load the resonance frequencies calculated using Workbench
%load('../ansys/apdl/ansys_workbench_res_freqs.txt');

%%%
% Plot the results
%figure
%set(gca,'fontsize',16)
%h1=plot(    [1:number_of_modes],cavity.res_freqs(:,1),'o',  ...
%            [1:length(ansys_resfreqs)],ansys_resfreqs,'.',  ...
%            [1:length(ansys_workbench_res_freqs)],          ...
%                    ansys_workbench_res_freqs(:,3),'x'          ...
%                );
%set(h1(:),'linewidth',2)
%set(h1(1),'markersize',10)
%set(h1(2),'markersize',15)
%set(h1(3),'markersize',10)
%xlabel('Mode Index');
%ylabel('Resonance Frequency [Hz]')
%title('Resonance Frequencies of Cavity');
%l1=legend('Matlab','Ansys MAPDL','Ansys Workbench','location','SE');
%grid

%% Compare the SPLs at a point calculated using Ansys and Matlab
% These next commands will load the results from 
%
% * Ansys Workbench analysis where a |mass source| was applied using the
%   ACT Acoustics extension, that uses the APDL command
%   |BF,node,JS,masssource|. The sound pressure level is stored in the
%   file.
% * Ansys Mechanical APDL, where a |mass source| was applied using the
%   the APDL command |BF,node,JS,masssource|, and hence this result should
%   be identical the results calculated using Ansys Workbench. The sound
%   pressure level is stored in the results file.
% 
% and then the Ansys results will be compared with the theoretical results
% calculated using this Matlab script.
%
% This script calculate the sound pressure which has to be converted into a
% sound pressure level (SPL) using the expression
%
% $$
% \textrm{SPL} = 20 \, \log_{10} 
%   \left[
%       \frac{p_{\textrm{matlab}}}{20 \times 10^{-6} \times \sqrt{2}}
%   \right] \quad \textrm{dB re 20$\mu$Pa}
% $$
%
% where $p_{\textrm{matlab}}$ is the pressure calculated using this script,
% $20 \times 10^{-6}$ is the reference pressure and the $\sqrt{2}$ is used
% to convert the peak levels calculated in the Matlab script to RMS levels.
%
% Load the Ansys results
%load('ansys_wb_full_harm_spl_receiver.txt');
%load('ansys_FULL_SPL_mic.txt');

%%%
% Plot the results
%figure
%set(gca,'fontsize',16)
%h2=plot(freq,20*log10(abs(pressure)/20e-6/sqrt(2)),     ...
%            ansys_wb_full_harm_spl_receiver(:,1),   ...
%                ansys_wb_full_harm_spl_receiver(:,2),'--',   ...
%            ansys_FULL_SPL_mic(:,1),ansys_FULL_SPL_mic(:,2),':');
%set(h2(:),'linewidth',2)
%set(h2(2),'markersize',15)
%xlabel('Frequency [Hz]')
%ylabel('SPL [dB re 20 \mu Pa]')
%title('SPL at a point within a rigid walled cavity')
%l2=legend('Matlab','Ansys: Workbench','Ansys: MAPDL','location','SE');
%grid


%% Compare the pressure at a point calculated using Ansys and Matlab
% The real and imaginary parts of the sound pressure (not SPL in dB) 
% was also calculated using Ansys Mechanical APDL using an acoustic mass
% source using the MAPDL command |BF,node,JS,masssource|, and also using
% the older style of acoustic load |F,node,FLOW,flowload|, where a modal
% superposition |MSUP| harmonic analysis was conducted.
%
% The |FLOW| load will apply an acoustic mass volume _acceleration_ whereas
% the mass source will applly an acoustic mass volume _velocity_. 
%
% The results from applying a |FLOW| load can be converted into results 
% as if a volume velocity had been applied, by multiplying the results by
% $j \omega$.

%load('../ansys/apdl/ansys_FULL_p_receiver.txt');
%ansys_FULL_pressure =   ansys_FULL_p_receiver(:,2)      ...
%                        + 1i .* ansys_FULL_p_receiver(:,3);
%ansys_FULL_freq = ansys_FULL_p_receiver(:,1);
                    

% Load in the undamped data
load('../ansys/apdl/ansys_MSUP_p_receiver_undamped.txt');
ansys_MSUP_p_receiver = ansys_MSUP_p_receiver_undamped;
ansys_MSUP_pressure =   ansys_MSUP_p_receiver(:,2)    ...
                        +1i.*ansys_MSUP_p_receiver(:,3);
ansys_MSUP_freq = ansys_MSUP_p_receiver(:,1);

% The Ansys modal super-position results were calculated using a FLOW load,
% so it is necessary to multiply the pressure results by j*omega to convert
% it from a volume acceleration to a volume velocity source.
ansys_MSUP_pressure = ansys_MSUP_pressure.* (1i*2*pi*ansys_MSUP_freq);
ansys_MSUP_pressure_undamped = ansys_MSUP_pressure;


% Load in the DMPRAT data
load('../ansys/apdl/ansys_MSUP_p_receiver_DMPRAT.txt');
ansys_MSUP_p_receiver = ansys_MSUP_p_receiver_DMPRAT;
ansys_MSUP_pressure =   ansys_MSUP_p_receiver(:,2)    ...
                        +1i.*ansys_MSUP_p_receiver(:,3);
ansys_MSUP_freq = ansys_MSUP_p_receiver(:,1);

% The Ansys modal super-position results were calculated using a FLOW load,
% so it is necessary to multiply the pressure results by j*omega to convert
% it from a volume acceleration to a volume velocity source.
ansys_MSUP_pressure = ansys_MSUP_pressure.* (1i*2*pi*ansys_MSUP_freq);
ansys_MSUP_pressure_DMPRAT = ansys_MSUP_pressure;



% Load in the ALPHAD and BETAD data
load('../ansys/apdl/ansys_MSUP_p_receiver_Rayleigh.txt');
ansys_MSUP_p_receiver = ansys_MSUP_p_receiver_Rayleigh;
ansys_MSUP_pressure =   ansys_MSUP_p_receiver(:,2)    ...
                        +1i.*ansys_MSUP_p_receiver(:,3);
ansys_MSUP_freq = ansys_MSUP_p_receiver(:,1);

% The Ansys modal super-position results were calculated using a FLOW load,
% so it is necessary to multiply the pressure results by j*omega to convert
% it from a volume acceleration to a volume velocity source.
ansys_MSUP_pressure = ansys_MSUP_pressure.* (1i*2*pi*ansys_MSUP_freq);
ansys_MSUP_pressure_Rayleigh = ansys_MSUP_pressure;



% Load in the MDAMP data
load('../ansys/apdl/ansys_MSUP_p_receiver_MDAMP.txt');
ansys_MSUP_p_receiver = ansys_MSUP_p_receiver_MDAMP;
ansys_MSUP_pressure =   ansys_MSUP_p_receiver(:,2)    ...
                        +1i.*ansys_MSUP_p_receiver(:,3);
ansys_MSUP_freq = ansys_MSUP_p_receiver(:,1);

% The Ansys modal super-position results were calculated using a FLOW load,
% so it is necessary to multiply the pressure results by j*omega to convert
% it from a volume acceleration to a volume velocity source.
ansys_MSUP_pressure = ansys_MSUP_pressure.* (1i*2*pi*ansys_MSUP_freq);
ansys_MSUP_pressure_MDAMP = ansys_MSUP_pressure;


%% Plot results 
figure
set(gca,'fontsize',14)
hp3 = subplot(2,1,1)
set(hp3,'fontsize',14)
%            freq,20*log10(abs(pressure)/20e-6/sqrt(2))-120, 'k', ...
h3=plot(     ...
            ansys_MSUP_freq,20*log10(abs(ansys_MSUP_pressure_undamped)), 'k', ...
            ansys_MSUP_freq,20*log10(abs(ansys_MSUP_pressure_DMPRAT)), 'k--', ...
            ansys_MSUP_freq,20*log10(abs(ansys_MSUP_pressure_Rayleigh)), 'k-.', ...
            ansys_MSUP_freq,20*log10(abs(ansys_MSUP_pressure_MDAMP)),'k:');
set(h3(:),'linewidth',2)
set(h3(1),'color',[0.5 0.5 0.5])
set(h3(2),'color',0.5*ones(1,3))
xlabel('Frequency [Hz]')
ylabel('Amplitude [dB re 1 Pa/(m^3/s)]')
title('Pressure at a point within a rigid-walled cavity')
grid
axis([0 500 20 120])
l3=legend('ANSYS: Undamped','ANSYS: DMPRAT','ANSYS: Rayleigh','ANSYS: MDAMP','location','SW');
legend(l3,'boxoff')
set(l3,'fontsize',12)


h3 = subplot(2,1,2)
set(h3,'fontsize',14)
%            freq,angle(pressure)/pi*180, 'c', ...
h3=plot(     ...
            ansys_MSUP_freq,angle(ansys_MSUP_pressure_undamped)/pi*180, 'k', ...
            ansys_MSUP_freq,angle(ansys_MSUP_pressure_DMPRAT)/pi*180, 'k--',...
            ansys_MSUP_freq,angle(ansys_MSUP_pressure_Rayleigh)/pi*180, 'k-.',...
            ansys_MSUP_freq,angle(ansys_MSUP_pressure_MDAMP)/pi*180,'k:');
set(h3(:),'linewidth',2)
set(h3(1),'color',[0.5 0.5 0.5])
set(h3(2),'color',0.5*ones(1,3))
xlabel('Frequency [Hz]')
ylabel('Phase Angle [Degrees]')

grid

print -depsc 'rigid_walled_cavity_damped.eps'

% Zoom in on the first resonance peak
figure
set(gca,'fontsize',16)
h3=plot(     ...
            ansys_MSUP_freq,20*log10(abs(ansys_MSUP_pressure_undamped)), 'k', ...
            ansys_MSUP_freq,20*log10(abs(ansys_MSUP_pressure_DMPRAT)), 'k--', ...
            ansys_MSUP_freq,20*log10(abs(ansys_MSUP_pressure_Rayleigh)), 'k-.', ...
            ansys_MSUP_freq,20*log10(abs(ansys_MSUP_pressure_MDAMP)),'k:');
set(h3(:),'linewidth',2)
set(h3(1),'color',0.5*ones(1,3))
set(h3(2),'color',0.5*ones(1,3))
xlabel('Frequency [Hz]')
ylabel('Amplitude [dB re 1 Pa/(m^3/s)]')
title('Pressure at a point within a rigid-walled cavity')
l3=legend('ANSYS: Undamped','ANSYS: DMPRAT','ANSYS: Rayleigh','ANSYS: MDAMP','location','NW');
grid
axis([152 160 85 95])

% Locate freqs down 3dB
max_pres = max(abs(ansys_MSUP_pressure_DMPRAT([150*2:1:170*2])));
f_res = find(abs(ansys_MSUP_pressure_DMPRAT)==max_pres)/2;

% Locate frequencies at the 3dB bandwidth
high_indices = find(abs(ansys_MSUP_pressure_DMPRAT([150*2:1:170*2]))>=max_pres/sqrt(2))+150*2-1;
f1 = interp1(abs(ansys_MSUP_pressure_DMPRAT(min(high_indices)-1:min(high_indices))), ...
    ansys_MSUP_freq(min(high_indices)-1:min(high_indices)), ...
    max_pres/sqrt(2))
f2 = interp1(abs(ansys_MSUP_pressure_DMPRAT(max(high_indices): 1+max(high_indices))), ...
    ansys_MSUP_freq(max(high_indices): 1 + max(high_indices)), ...
    max_pres/sqrt(2))
% Estimated damping ratio
xi_1 = 0.5*(f2-f1)/f_res

hold
h4 = plot(f_res,20*log10(max_pres),'k*', ...
    f1,20*log10(max_pres/sqrt(2)),'k*',f2,20*log10(max_pres/sqrt(2)),'k*')
set(h4(:),'linewidth',2)
print -depsc 'rigid_walled_cavity_damped_zoom.eps'