%% Room Acoustics
% This file contains the analysis of the random incidence sound absorption
% coefficient obtained from a reverberation chamber.
% 
% Calculations are based on theory as well as imported results from both
% Ansys Classic and Ansys Workbench.
%
% Note: If you click on File > Save and Publish to HTML you will have a
% nice record of this tutorial and its outputs.
%
% References:
% 1. Bies, D. and Hansen, C.H. (2009), Engineering Acoustics, 4th Edn.
% 2. Jacobsen, F. (2011), The sound field in a reverberation room, Note no 31261
%       Technical University of Denmark
%
%
% Author: BSC
% Date: 6/1/2013
%
% Revision History
% Date      Change 
% 

close all
clear all
clc


%% System parameters
%  Define the important parameters used in the analysis.

% Room properties
Lx = 5.0                                % [m] Depth of room
Ly = 7.0                                % [m] Width of room
Lz = 6.0                                % [m] Height of room

% Absorbent properties
Span = 3.0                              % [m] Span of absorbing layer
Alpha_1 = 1                             % Sound absorption coefficient

% Properties of the air
% Default values used in Ansys Workbench
%c_0 = 343.24                            % [m/s] Speed of sound in air
%rho_0 = 1.2041                          % [kg/m^3] Density of air
% Values chosen for this example
c_0 = 344                               % [m/s] Speed of sound in air
rho_0 = 1.21                            % [kg/m^3] Density of air

% Analysis options
fs = 500                                % [Hz] Transient analysis sample rate
fc = 125                                % [Hz] Octave band centre frequency
fc_low = 2^(-1/2)*fc                    % [Hz] Lower bound of octave band
fc_high = 2^(1/2)*fc                    % [Hz] Upper bound of octave band
%fc_low = 2^(-1/6)*fc                    % [Hz] Lower bound of third-octave band
%fc_high = 2^(1/6)*fc                    % [Hz] Upper bound of third-octave band
lambda_c = c_0/fc                       % [m] Wavelength at centre frequency



%% Theoretical Analysis
% The following cell has been used to calculate the expected room response
% based on theory.
%
% The Reverberation Time is given by [Bies and Hansen] 
%
% $$ T_{60} = \frac{55.25*V}{c_0*S_{\alpha}*\alpha_1} $$
%

S_alpha = Span.^2                       % [m^2] Area of absorbing material
S = 2*(Lx*Lz + Lx*Ly + Ly*Lz)           % [m^2] Area of walls of room
L = 4*(Lx + Ly + Lz)                    % [m] Perimeter of walls of room
V = Lx*Ly*Lz                            % [m^3] Volume of room

% Expected reverberation time.  Bies and Hansen Eq. (7.52)
T60 = 55.25*V/(c_0*S_alpha*Alpha_1)     % [s] 

% Number of modes.  Bies and Hansen Eq. (7.21)
N = 4*pi*fc^3*V/(3*c_0^3) + pi*fc^2*S/(4*c_0^2) + L*fc/(8*c_0)
% Number of modes in the band
N_band = 4*pi*fc_high^3*V/(3*c_0^3) + pi*fc_high^2*S/(4*c_0^2) + L*fc_high/(8*c_0) ...
    - (4*pi*fc_low^3*V/(3*c_0^3) + pi*fc_low^2*S/(4*c_0^2) + L*fc_low/(8*c_0))

% Modal density.  Bies and Hansen Eq. (7.22)
dNdf = 4*pi*fc^2*V/(c_0^3) + pi*fc*S/(2*c_0^2) + L/(8*c_0)
delta_f = 2.2/T60                       % [Hz] Average bandwidth
disp('Modal density across the band')
N_band/(fc_high-fc_low)

% Modal overlap.  Bies and Hansen Eq. (7.25)
M = delta_f*dNdf

% Restriction on wavelength and frequency for room
% From Bies and Hansen (Section 6.6.2)
lambda_max =(V/1.3)^(1/3)               % [m] lambda must be less than this
f_min = c_0/lambda_max                  % [Hz] Frequency should be greater than this

% Estimate the sound power radiated by a source in a diffuse field
Mass_Source = 1     % Amplitude of the mass source in Ansys
Q = Mass_Source/rho_0       % Source volume velocity
% Expected sound power level for a pure tone at centre freq. Based on Jacobsen Eq. (3.28) 
Lw_estimated = 10*log10(rho_0*c_0*((2*pi*fc)/c_0)^2*Q^2/(8*pi))+120 % Pure tone
f_range = [fc_low:1:fc_high];      % Create a frequency vector over the octave band
Lw_estimated_band = 10*log10(sum(rho_0*c_0*((2*pi*f_range)/c_0).^2*Q^2/(8*pi)))+120         % Summed over an octave band
% Expected standard deviation for the tones.  Jacobsen Eq. (3.30)
sqrt(2)*inv(sqrt(M*pi))
10*log10(sqrt(2)*rho_0*c_0*((2*pi*fc)/c_0)^2*Q^2/(8*pi)*(1+inv(sqrt(M*pi)))) +120 - ...
    Lw_estimated 
% Expected standard dev in Lw for bandwidth.  Jacobsen Eq. (3.31)
inv(2*pi*fc/c_0) * sqrt(2*pi*c_0/(V*(fc_high-fc_low)) )



%% Natural frequencies
%  Calculate the undamped natural frequencies of the room
Modal_vector = [];      % Predefine results matrix
Max_index = 10;         % The maximum number of modal indices to search through
for nx = 0:Max_index            % x modal index
    for  ny = 0:Max_index       % y modal index
        for nz = 0:Max_index    % z modal index
            % Calculate the natural frequency from Bies and Hansen, Eq. (7.17)
            nat_freq = c_0/2*sqrt( (nx/Lx)^2 + (ny/Ly)^2 +(nz/Lz)^2 );
            Modal_vector = [Modal_vector;[nx,ny,nz,nat_freq]];
        end
    end
end

% Sort the vector from lowest to highest
[Y,I] = sort(Modal_vector(:,4));        % Determine the index
Modal_vector = Modal_vector(I,:);       % Use the index to sort
% Add a column increasing from 1 to length of vector
Modal_vector = [ [1:length(Modal_vector)]',Modal_vector];    
disp('Display the modes below the octave band upper limit')
modes_of_interest = find(Modal_vector(:,5)<fc_high);
length(modes_of_interest)   % Total number of modes below upper freq limit
Modal_vector(modes_of_interest,:)

disp('Determine the number of modes in the octave band')
modes_of_interest = find(Modal_vector(:,5)<fc_high & ...
                         Modal_vector(:,5)>fc_low );       
number_of_modes_of_interest = length(modes_of_interest)       

% Actual modal density across the band is therefore


%% Load in Ansys Parameters
%  Parameters used in the Ansys analysis have been exported as a permanent
%  record.  Note that the last 7 items are for the last frequency step
%  of the harmonic analysis and are used as a check that the vector of 
%  values was exported correctly.
disp('Ansys Analysis')
load('../ansys/apdl/Sabine_parameters.txt')
Density_Ansys = Sabine_parameters(1)        % Density of air
Speed_Sound_Ansys = Sabine_parameters(2)    % Speed of sound
LEN_Ansys = Sabine_parameters(3)            % Room length
WIDTH_Ansys = Sabine_parameters(4)          % Room width
HGT_Ansys = Sabine_parameters(5) 			% Room height
SPAN_Ansys = Sabine_parameters(6) 			% Span of absorbent
Alpha_Ansys = Sabine_parameters(7) 			% Sound absorption coefficient
Fs_Ansys = Sabine_parameters(8)      		% Sample rate for transient analysis
Fc_Ansys = Sabine_parameters(9)  			% Centre frequency for harmonic analysis
Mesh_size = Sabine_parameters(10)  			% Mesh size
Pres_imag_flow = Sabine_parameters(11);		% Imaginary pressure of flow node
SPL_Sensor_Node_1 = Sabine_parameters(12);  % SPL of mic 1
SPL_Sensor_Node_2 = Sabine_parameters(13);  % SPL of mic 2
SPL_Sensor_Node_3 = Sabine_parameters(14);  % SPL of mic 3
SPL_Sensor_Node_4 = Sabine_parameters(15);  % SPL of mic 4
SPL_Sensor_Node_5 = Sabine_parameters(16);  % SPL of mic 5
SPL_Sensor_Node_6 = Sabine_parameters(17);  % SPL of mic 6


% Calculate the mean SPL of the 6 microphone locations
SPL_mean = 10*log10( 10^(SPL_Sensor_Node_1/10) ...
    + 10^(SPL_Sensor_Node_2/10) ...
    + 10^(SPL_Sensor_Node_3/10) ...
    + 10^(SPL_Sensor_Node_4/10) ...
    + 10^(SPL_Sensor_Node_5/10) ...
    + 10^(SPL_Sensor_Node_6/10)  ) - 10*log10(6);

% Number of elements / wavelength.  Calculated at the octave band centre
% frequency (namely 100Hz).  Should be > 6 for quadratic elements and > 12
% for linear elements.
EPW = lambda_c/Mesh_size





%% Modal Analysis
%  This cell reads in the results from the modal analysis conducted in 
%  Ansys.  It reads both the undamped and damped results.
sabine_modal_classic_undamped = textread('../ansys/apdl/sabine_modal_classic_undamped.txt','','headerlines',5,'delimiter',' ');
sabine_modal_classic_damped = textread('../ansys/apdl/sabine_modal_classic_damped.txt','','headerlines',5,'delimiter',' ');
Modal_vector_undamped = sabine_modal_classic_undamped(:,1:2);


% Extract information from damped analysis
damp_len = length(sabine_modal_classic_damped(:,2));    % Num rows in matrix
% Convert to a complex eigenvalue
Modal_vector_damped = [1,j]*reshape(sabine_modal_classic_damped(:,2),2,damp_len/2);
% Rewrite as [damping ratio, natural frequency]
Modal_vector_damped = [real(Modal_vector_damped)./abs(Modal_vector_damped); ...
                       abs(Modal_vector_damped)];
% Select every second element since there are complex conjugates
Modal_vector_damped = Modal_vector_damped(:,1:2:damp_len/2)';
% Add an index column
Modal_vector_damped = [[1:damp_len/4]',Modal_vector_damped];

% Tabulate results
disp('_____________________________________')
disp('Comparison of parameters in Workbench')
disp(['Mode        ',    'Matlab   ','Ansys Undamp   ','Ansys Damp'])
N_modes = 170;      % Number of modes to tabulate
[Modal_vector(1:N_modes,1), Modal_vector(1:N_modes,5), ...
    Modal_vector_undamped(1:N_modes,2), Modal_vector_damped(1:N_modes,3)   ]





%% Steady state harmonic analysis
%  This cell reads in the results from the harmonic analysis conducted in
%  Ansys.  
%
%  The sound power from the source may be calculated for either a FLOW
%  source or a Mass source.  The user must select which.
%
%  It then uses the following expression to calculate the 
%  reverberation time from the mean sound pressure level and the sound
%  power level from the source.
% 
% $$T_{60} = 10^{( (L_p - L_w + 10\log_{10}(V) + 10\log_{10}(1+S\lambda_c/(8V))-13.5 )/10 )}$$
%
% 
load('../ansys/apdl/Sabine_harm.txt')
fsteps = size(Sabine_harm,1)            % Number of frequency steps
Fc_Ansys = Sabine_harm(:,1);            % Frequencies for the harmonic analysis
Pres_imag_flow = Sabine_harm(:,2);      % [Pa] Imaginary pressure at source node
SPL_Sensor_Nodes = Sabine_harm(:,3);    % [dB re 20uPa] Mean SPL of microphones

% Sound power level of source
%source_type = 'FLOW'
source_type = 'MASS'
switch source_type
    % Calculate the sound power level for each frequency
    case 'FLOW'
        % [dB re 10^-12W] Sound power for FLOW source. 
        % Note have assumed a unit real source
        Lw = 10*log10(1./(2*pi*Fc_Ansys)/Density_Ansys.*abs(-Pres_imag_flow))+120-3; 
    case 'MASS'
        % [dB re 10^-12W] Sound power for Mass Source.  
        % Note have assumed a unit imaginary source
        Lw = 10*log10(1./Density_Ansys.*abs(Pres_imag_flow))+120-3;
    otherwise
        warning('Unexpected source type.');
end

Lw_total = 10*log10( sum(10.^(Lw/10)) ) % [dB re 10^-12W] Total sound power in the band.  

% Sound pressure levels dB re 20uPa
Lp = SPL_Sensor_Nodes;                  % [dB re 20uPa] Mean SPL of the 6 mics for each frequency
Lp_total = 10*log10( sum(10.^(Lp/10)) ) % [dB re 20uPa] Total SPL for the entire frequency band

% Calculate the reverberation time from SPL and Lw.
% From Bies and Hansen, Eq. (6.13)
% Lw = Lp + 10log10(V)-10log10(T60) + 10log10(1+S*lambda/(8V)-13.5
T60_ss = 10^( (Lp_total - Lw_total + 10*log10(V) + 10*log10(1+S*lambda_c/(8*V))-13.5 )/10 )
% From the reverberation time calculate the absorption coefficient
Alpha_ss = 55.25*V/(c_0*S_alpha*T60_ss) 


% Plot the results
figure
plot(Fc_Ansys, Lp,'k', 'LineWidth',2)
hold on
plot(Fc_Ansys, Lw, 'k--', 'LineWidth',2)
hold off
set(gcf,'Name','Harmonic Response')
set(gca, 'FontSize', 16)        % Increase font for printing
legend('Mean SPL dB re 20\muPa','Sound Power Level dB re 10^{-12}W', ...
       'Location','Best')
title('Harmonic Response')
xlabel('Frequency (Hz)')
ylabel('Amplitude dB')
grid
print -depsc 'harmonic_response.eps'




%% Load in Ansys Transient Analysis Results
%  This cell reads in the transient analysis results.
%
%  There are  7 columns.  The first is the time vector, followed by the
%  pressures at the 6 microphone locations.

load('../ansys/apdl/Sabine_trans.txt')
T = Sabine_trans(:,1);          % [s] Time vector
P1 = Sabine_trans(:,2);         % [Pa] Pressure mic 1
P2 = Sabine_trans(:,3);         % [Pa] Pressure mic 2
P3 = Sabine_trans(:,4);         % [Pa] Pressure mic 3
P4 = Sabine_trans(:,5);         % [Pa] Pressure mic 4
P5 = Sabine_trans(:,6);         % [Pa] Pressure mic 5
P6 = Sabine_trans(:,7);         % [Pa] Pressure mic 6

Num_samples = length(T)         % Number of samples


%% Process Ansys Transient Results
%  This cell takes the Ansys transient data, applies an ANSI standard 
%  octave band filter to the pressure response of the 6 microphones.  

% Design Butterworth 2Nth-order third-octave filter 
% Note: BUTTER is based on a bilinear transformation, as suggested in
%    [1] ANSI S1.1-1986 (ASA 65-1986): Specifications for
%        Octave-Band and Fractional-Octave-Band Analog and
%        Digital Filters, 1993.
if (fc_high > (fs/2))            % Check if poles too close to Nyquist freq
  error('Design not possible. Check frequencies.');
end
N_order = 3;                    % Filter order
Qr = fc/(fc_high - fc_low);     
Qd = (pi/2/N_order)/(sin(pi/2/N_order))*Qr;
alpha = (1 + sqrt(1+4*Qd^2))/2/Qd; 
W1 = fc/(fs/2)/alpha;           % Lower bound for filter
W2 = fc/(fs/2)*alpha;           % Upper bound for filter
[Bc,Ac] = butter(N_order,[W1,W2]); % Filter coefficients

% Plot the frequency response of the filter to make sure ok
figure                          % Open a new figure
set(gcf,'Name','FRF of the Octave Band Filter')
Lz = freqz(Bc,Ac,[0:fs/2],fs);       
plot([0:fs/2],20*log10(abs(Lz)),'k','LineWidth',2)
hold
% Plot the lower and upper bounds of the octave band
plot([fc_low,fc_high],20*log10(abs(freqz(Bc,Ac,round([fc_low,fc_high]),fs))),'ko','LineWidth',2)
set(gca, 'FontSize', 16)        % Increase font for printing
title(['Frequency response of ',num2str(fc),' Hz octave band filter'])
xlabel('Frequency (Hz)')
ylabel('Magnitude (dB)')
grid                            % Add a grid
axis([0 250 -60 0])             % Bound the axes
print -depsc 'octave_filter.eps'

% Filter pressures for desired octave band
P1 = filtfilt(Bc,Ac,P1);
P2 = filtfilt(Bc,Ac,P2);
P3 = filtfilt(Bc,Ac,P3);
P4 = filtfilt(Bc,Ac,P4);
P5 = filtfilt(Bc,Ac,P5);
P6 = filtfilt(Bc,Ac,P6);

% Plot the filtered pressures
figure                                  % Open a new figure
plot(T,P1,T,P2,T,P3,T,P4,T,P5,T,P6)     % Plot the filtered pressures
plot(T,[P1,P2,P3,P4,P5,P6],'LineWidth',2)     % Plot the filtered pressures
set(gca, 'FontSize', 16)                % Increase font for printing
xlabel('Time (s)')
ylabel('Pressure (Pa)')
title('Pressure response at the 6 microphone locations')
grid
print -depsc 'octave_filtered_pressures.eps'


% Instantaneous sound pressure levels
SPL1 = 20*log10(abs(P1)/20E-06);
SPL2 = 20*log10(abs(P2)/20E-06);
SPL3 = 20*log10(abs(P3)/20E-06);
SPL4 = 20*log10(abs(P4)/20E-06);
SPL5 = 20*log10(abs(P5)/20E-06);
SPL6 = 20*log10(abs(P6)/20E-06);

% Run a smoothing filter over the SPLs
SPL1_B = conv(SPL1',ones(10,1)/10);
SPL2_B = conv(SPL2',ones(10,1)/10);
SPL3_B = conv(SPL3',ones(10,1)/10);
SPL4_B = conv(SPL4',ones(10,1)/10);
SPL5_B = conv(SPL5',ones(10,1)/10);
SPL6_B = conv(SPL6',ones(10,1)/10);

% Calculate the linear regression
% Samples to conduct regression over.  Skip first 1sec of the transient
Regression_Span = [Num_samples*0.2:Num_samples];
% Generate the RH input matrix
X = [T(Regression_Span).^0,T(Regression_Span)]';
% Calculate the linear constants
K1 = SPL1_B(Regression_Span)*pinv(X);
K2 = SPL2_B(Regression_Span)*pinv(X);
K3 = SPL3_B(Regression_Span)*pinv(X);
K4 = SPL4_B(Regression_Span)*pinv(X);
K5 = SPL5_B(Regression_Span)*pinv(X);
K6 = SPL6_B(Regression_Span)*pinv(X);

% Calculate the Regression coefficient, R^2. Unity is best
R1 = 1- mean((SPL1_B(Regression_Span)- K1*X).^2) ...
    ./ mean((SPL1_B(Regression_Span)- mean(SPL1_B(Regression_Span)) ).^2)
R2 = 1- mean((SPL2_B(Regression_Span)- K2*X).^2) ...
    ./ mean((SPL2_B(Regression_Span)- mean(SPL2_B(Regression_Span)) ).^2)
R3 = 1- mean((SPL3_B(Regression_Span)- K3*X).^2) ...
    ./ mean((SPL3_B(Regression_Span)- mean(SPL3_B(Regression_Span)) ).^2)
R4 = 1- mean((SPL4_B(Regression_Span)- K4*X).^2) ...
    ./ mean((SPL4_B(Regression_Span)- mean(SPL4_B(Regression_Span)) ).^2)
R5 = 1- mean((SPL5_B(Regression_Span)- K5*X).^2) ...
    ./ mean((SPL5_B(Regression_Span)- mean(SPL5_B(Regression_Span)) ).^2)
R6 = 1- mean((SPL6_B(Regression_Span)- K6*X).^2) ...
    ./ mean((SPL6_B(Regression_Span)- mean(SPL6_B(Regression_Span)) ).^2)


% Calculate the T60 for each microphone
T60_Ansys_1 = -60/K1(2)
T60_Ansys_2 = -60/K2(2)
T60_Ansys_3 = -60/K3(2)
T60_Ansys_4 = -60/K4(2)
T60_Ansys_5 = -60/K5(2)
T60_Ansys_6 = -60/K6(2)

% Mean T60
T60_Ansys_mean = inv(mean([1/T60_Ansys_1,1/T60_Ansys_2,1/T60_Ansys_3,...
    1/T60_Ansys_4,1/T60_Ansys_5,1/T60_Ansys_6]))
% Effective Sabine absorption coefficient
Alpha_Ansys_mean = 55.25*V/(c_0*S_alpha*T60_Ansys_mean)



%% Plot the decay and fit
figure
X = [T.^0,T]';                  % Redefine the RH input matrix
set(gcf,'Name','Decay plots')
subplot(3,2,1)                  % Plot mic 1
plot(T, SPL1_B(1:Num_samples),...  % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K1*X, ...                % Data fit
    'k', 'LineWidth',2)         % Set line width to 2
hold off
set(gca, 'FontSize', 14)        % Increase font for printing
axis([0, 5, -70, 50])           % Set axis
title('Microphone 1')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_1*10)/10),'s'])


subplot(3,2,2)                  % Plot mic 2
plot(T, SPL2_B(1:Num_samples),...   % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K2*X, ...                % Data fit
    'k','LineWidth',2)          % Set line width to 2
hold off
set(gca, 'FontSize', 14)        % Increase font for printing
axis([0, 5, -70, 50])           % Set axis
title('Microphone 2')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_2*10)/10),'s'])


subplot(3,2,3)                  % Plot mic 3
plot(T, SPL3_B(1:Num_samples),...    % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K3*X, ...                % Data fit
    'k', 'LineWidth',2)         % Set line width to 2
hold off
set(gca, 'FontSize', 14)        % Increase font for printing
axis([0, 5, -70, 50])           % Set axis
title('Microphone 3')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_3*10)/10),'s'])


subplot(3,2,4)                  % Plot mic 4
plot(T, SPL4_B(1:Num_samples),...    % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K4*X, ...                % Data fit
    'k','LineWidth',2)          % Set line width to 2
hold off
set(gca, 'FontSize', 14)        % Increase font for printing
axis([0, 5, -70, 50])           % Set axis
title('Microphone 4')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_4*10)/10),'s'])


subplot(3,2,5)                  % Plot mic 5
plot(T, SPL5_B(1:Num_samples),...   % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K5*X, ...                % Data fit
    'k', 'LineWidth',2)  % Set line width to 2
hold off
set(gca, 'FontSize', 14)        % Increase font for printing
axis([0, 5, -70, 50])           % Set axis
title('Microphone 5')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_5*10)/10),'s'])


subplot(3,2,6)                  % Plot mic 6
plot(T, SPL6_B(1:Num_samples),...  % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K6*X, ...                % Data fit
    'k', 'LineWidth',2)         % Set line width to 2
hold off
set(gca, 'FontSize', 14)        % Increase font for printing
axis([0, 5, -70, 50])           % Set axis
title('Microphone 6')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_6*10)/10),'s'])

print -depsc 'reverberation_times.eps'




%% Tabulate scalar parameters
disp('________________________________________')
disp('Comparison of parameters from Ansys APDL')
disp(['Parameter        ','Matlab   ','Ansys    ','% Diff'])
disp(['T60, Harmonic    ',num2str(T60),'  ',num2str(T60_ss),'  ', ...
    num2str(abs(T60_ss-T60)/T60*100)  ])
disp(['T60, Transient   ',num2str(T60),'  ',num2str(T60_Ansys_mean),'  ', ...
    num2str(abs(T60_Ansys_mean-T60)/T60*100)  ])

disp(['Alpha, Harmonic  ',num2str(Alpha_1),'  ',num2str(Alpha_ss),'  ', ...
    num2str(abs(Alpha_1 - Alpha_ss)/Alpha_1*100)])
disp(['Alpha, Transient ',num2str(Alpha_1),'  ',num2str(Alpha_Ansys_mean),'  ', ...
    num2str(abs(Alpha_1 - Alpha_Ansys_mean)/Alpha_1*100)])









%% Workbench Results
disp('________________________________')
disp('Now repeat for Workbench Results')
disp('________________________________')



%% Workbench Modal Analysis
Workbench_Undamped = textread('../ansys/workbench/Sabine_Modal_Undamped.txt','','headerlines',1,'delimiter',' ');
Workbench_Damped = textread('../ansys/workbench/Sabine_Modal_Damped.txt','','headerlines',2,'delimiter',' ');
% Append row for bulk compression mode which was stripped out due to "N/A"
% Change damped frequency to natural frequency
Workbench_Damped(:,3) = Workbench_Damped(:,3)./sqrt(1-Workbench_Damped(:,5).^2);
Workbench_Damped = [[1 1 0 0 1 0 0] ; Workbench_Damped];

% Tabulate results
disp('_____________________________________')
disp('Comparison of parameters in Workbench')
disp(['Mode        ',    'Matlab   ','Ansys Undamp   ','Ansys Damp'])
N_count = 170;      % Limit table to this number of modes
[Modal_vector(1:N_count,1),Modal_vector(1:N_count,5),Workbench_Undamped(:,3),Workbench_Damped(:,3)   ]



%% Workbench Harmonic Analysis
% Load in the results
Source_Location = textread('../ansys/workbench/Sabine_Harm_Source_Pres.txt','','headerlines',17,'delimiter',' ');
Mic1 = textread('../ansys/workbench/Sabine_Harm_Mic1_SPL.txt','','headerlines',17,'delimiter',' ');
Mic2 = textread('../ansys/workbench/Sabine_Harm_Mic2_SPL.txt','','headerlines',17,'delimiter',' ');
Mic3 = textread('../ansys/workbench/Sabine_Harm_Mic3_SPL.txt','','headerlines',17,'delimiter',' ');
Mic4 = textread('../ansys/workbench/Sabine_Harm_Mic4_SPL.txt','','headerlines',17,'delimiter',' ');
Mic5 = textread('../ansys/workbench/Sabine_Harm_Mic5_SPL.txt','','headerlines',17,'delimiter',' ');
Mic6 = textread('../ansys/workbench/Sabine_Harm_Mic6_SPL.txt','','headerlines',17,'delimiter',' ');


Fc_Ansys = Mic1(:,1);                   % Frequencies for the harmonic analysis
fsteps = length(Fc_Ansys)               % Number of frequency steps
Pres_real_flow = Source_Location(:,2);  % Real pressure of source node
Pres_imag_flow = Source_Location(:,3);  % Imaginary pressure of source node

% Extract the SPLs
SPL1 = Mic1(:,2);
SPL2 = Mic2(:,2);
SPL3 = Mic3(:,2);
SPL4 = Mic4(:,2);
SPL5 = Mic5(:,2);
SPL6 = Mic6(:,2);

% Calculate the mean SPL of the 6 microphone locations
SPL_Sensor_Nodes = 10*log10( 10.^(SPL1/10) ...
    + 10.^(SPL2/10) ...
    + 10.^(SPL3/10) ...
    + 10.^(SPL4/10) ...
    + 10.^(SPL5/10) ...
    + 10.^(SPL6/10)  ) - 10*log10(6);



% Sound power level of source
source_type = 'FLOW'
source_type = 'MASS'
switch source_type
    % Calculate the sound power level for each frequency
    case 'FLOW'
        % Sound power for FLOW source. Note have assumed a unit real source
        Lw = 10*log10(1./(2*pi*Fc_Ansys)/Density_Ansys.*abs(-Pres_imag_flow))+120-3; 
    case 'MASS'
        % Sound power for Mass Source.  Note have assumed a unit imaginary source
        Lw = 10*log10(1./Density_Ansys.*abs(Pres_imag_flow))+120-3;
    otherwise
        warning('Unexpected source type.');
end

Lw_total = 10*log10( sum(10.^(Lw/10)) ) % Total sound power in the band.  dB re 10^-12W

% Sound pressure levels dB re 20uPa
Lp = SPL_Sensor_Nodes;                  % Mean SPL of the 6 mics for each frequency
Lp_total = 10*log10( sum(10.^(Lp/10)) ) % Total SPL for the entire frequency band

% Calculate the reverberation time from SPL and Lw.
% From Bies and Hansen, Eq. (6.13)
% Lw = Lp + 10log10(V)-10log10(T60) + 10log10(1+S*lambda/(8V)-13.5
T60_ss = 10^( (Lp_total - Lw_total + 10*log10(V) + 10*log10(1+S*lambda_c/(8*V))-13.5 )/10 )
% From the reverberation time calculate the absorption coefficient
Alpha_ss = 55.25*V/(c_0*S_alpha*T60_ss) 


% Plot the results
figure
plot(Fc_Ansys, Lp,'k', 'LineWidth',2)
hold on
plot(Fc_Ansys, Lw, 'k--', 'LineWidth',2)
hold off
set(gcf,'Name','Harmonic Response')
set(gca, 'FontSize', 16)        % Increase font for printing
legend('Mean SPL dB re 20\mu Pa','Sound Power Level dB re 10^{-12}W', ...
       'Location','Best')
title('Harmonic Response')
xlabel('Frequency (Hz)')
ylabel('Amplitude dB')
grid
print -depsc 'harmonic_response_wb.eps'




%% Workbench Transient Analysis

% Load in the results
mic1 = textread('../ansys/workbench/Sabine_Trans_Mic1_Pres.txt','','headerlines',1,'delimiter',' ');
mic2 = textread('../ansys/workbench/Sabine_Trans_Mic2_Pres.txt','','headerlines',1,'delimiter',' ');
mic3 = textread('../ansys/workbench/Sabine_Trans_Mic3_Pres.txt','','headerlines',1,'delimiter',' ');
mic4 = textread('../ansys/workbench/Sabine_Trans_Mic4_Pres.txt','','headerlines',1,'delimiter',' ');
mic5 = textread('../ansys/workbench/Sabine_Trans_Mic5_Pres.txt','','headerlines',1,'delimiter',' ');
mic6 = textread('../ansys/workbench/Sabine_Trans_Mic6_Pres.txt','','headerlines',1,'delimiter',' ');

% Time vector
T_WB = mic1(:,2);

% Sound Pressure Levels
P1 = mic1(:,3);
P2 = mic2(:,3);
P3 = mic3(:,3);
P4 = mic4(:,3);
P5 = mic5(:,3);
P6 = mic6(:,3);


% Filter pressures for desired octave band
P1 = filtfilt(Bc,Ac,P1);
P2 = filtfilt(Bc,Ac,P2);
P3 = filtfilt(Bc,Ac,P3);
P4 = filtfilt(Bc,Ac,P4);
P5 = filtfilt(Bc,Ac,P5);
P6 = filtfilt(Bc,Ac,P6);

% Plot the filtered pressures
figure                                  % Open a new figure
plot(T_WB,P1,T_WB,P2,T_WB,P3,T_WB,P4,T_WB,P5,T_WB,P6)     % Plot the filtered pressures
plot(T_WB,[P1,P2,P3,P4,P5,P6],'LineWidth',2)     % Plot the filtered pressures
set(gca, 'FontSize', 16)        % Increase font for printing
xlabel('Time (s)')
ylabel('Pressure (Pa)')
title('Pressure response at the 6 microphone locations')
grid
print -depsc 'octave_filtered_pressures.eps'


% Instantaneous sound pressure levels
SPL1 = 20*log10(abs(P1)/20E-06);
SPL2 = 20*log10(abs(P2)/20E-06);
SPL3 = 20*log10(abs(P3)/20E-06);
SPL4 = 20*log10(abs(P4)/20E-06);
SPL5 = 20*log10(abs(P5)/20E-06);
SPL6 = 20*log10(abs(P6)/20E-06);

% Run a smoothing filter over the SPLs
SPL1_B = conv(SPL1',ones(10,1)/10);
SPL2_B = conv(SPL2',ones(10,1)/10);
SPL3_B = conv(SPL3',ones(10,1)/10);
SPL4_B = conv(SPL4',ones(10,1)/10);
SPL5_B = conv(SPL5',ones(10,1)/10);
SPL6_B = conv(SPL6',ones(10,1)/10);

% Calculate the linear regression
% Samples to conduct regression over.  Skip first 1sec of the transient
Regression_Span = [Num_samples*0.2:Num_samples];
% Generate the RH input matrix
X = [T(Regression_Span).^0,T(Regression_Span)]';
% Calculate the linear constants
K1 = SPL1_B(Regression_Span)*pinv(X);
K2 = SPL2_B(Regression_Span)*pinv(X);
K3 = SPL3_B(Regression_Span)*pinv(X);
K4 = SPL4_B(Regression_Span)*pinv(X);
K5 = SPL5_B(Regression_Span)*pinv(X);
K6 = SPL6_B(Regression_Span)*pinv(X);

% Calculate the Regression coefficient, R^2. Unity is best
R1 = 1- mean((SPL1_B(Regression_Span)- K1*X).^2) ...
    ./ mean((SPL1_B(Regression_Span)- mean(SPL1_B(Regression_Span)) ).^2)
R2 = 1- mean((SPL2_B(Regression_Span)- K2*X).^2) ...
    ./ mean((SPL2_B(Regression_Span)- mean(SPL2_B(Regression_Span)) ).^2)
R3 = 1- mean((SPL3_B(Regression_Span)- K3*X).^2) ...
    ./ mean((SPL3_B(Regression_Span)- mean(SPL3_B(Regression_Span)) ).^2)
R4 = 1- mean((SPL4_B(Regression_Span)- K4*X).^2) ...
    ./ mean((SPL4_B(Regression_Span)- mean(SPL4_B(Regression_Span)) ).^2)
R5 = 1- mean((SPL5_B(Regression_Span)- K5*X).^2) ...
    ./ mean((SPL5_B(Regression_Span)- mean(SPL5_B(Regression_Span)) ).^2)
R6 = 1- mean((SPL6_B(Regression_Span)- K6*X).^2) ...
    ./ mean((SPL6_B(Regression_Span)- mean(SPL6_B(Regression_Span)) ).^2)


% Calculate the T60 for each microphone
T60_Ansys_1 = -60/K1(2)
T60_Ansys_2 = -60/K2(2)
T60_Ansys_3 = -60/K3(2)
T60_Ansys_4 = -60/K4(2)
T60_Ansys_5 = -60/K5(2)
T60_Ansys_6 = -60/K6(2)

% Mean T60
T60_Ansys_mean = inv(mean([1/T60_Ansys_1,1/T60_Ansys_2,1/T60_Ansys_3,...
    1/T60_Ansys_4,1/T60_Ansys_5,1/T60_Ansys_6]))
% Effective Sabine absorption coefficient
Alpha_Ansys_mean = 55.25*V/(c_0*S_alpha*T60_Ansys_mean)



%% Plot the decay and fit
figure
X = [T.^0,T]';                  % Redefine the RH input matrix
set(gcf,'Name','Decay plots')
subplot(3,2,1)                  % Plot mic 1
plot(T, SPL1_B(1:Num_samples),...  % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K1*X, ...                % Data fit
    'k', 'LineWidth',2)         % Set line width to 2
hold off
set(gca, 'FontSize', 16)        % Increase font for printing
axis([0, 5, -50, 50])           % Set axis
title('Microphone 1')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_1*10)/10),'s'])


subplot(3,2,2)                  % Plot mic 2
plot(T, SPL2_B(1:Num_samples),...   % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K2*X, ...                % Data fit
    'k','LineWidth',2)          % Set line width to 2
hold off
set(gca, 'FontSize', 16)        % Increase font for printing
axis([0, 5, -50, 50])           % Set axis
title('Microphone 2')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_2*10)/10),'s'])


subplot(3,2,3)                  % Plot mic 3
plot(T, SPL3_B(1:Num_samples),...    % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K3*X, ...                % Data fit
    'k', 'LineWidth',2)         % Set line width to 2
hold off
set(gca, 'FontSize', 16)        % Increase font for printing
axis([0, 5, -50, 50])           % Set axis
title('Microphone 3')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_3*10)/10),'s'])


subplot(3,2,4)                  % Plot mic 4
plot(T, SPL4_B(1:Num_samples),...    % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K4*X, ...                % Data fit
    'k','LineWidth',2)          % Set line width to 2
hold off
set(gca, 'FontSize', 16)        % Increase font for printing
axis([0, 5, -50, 50])           % Set axis
title('Microphone 4')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_4*10)/10),'s'])


subplot(3,2,5)                  % Plot mic 5
plot(T, SPL5_B(1:Num_samples),...   % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K5*X, ...                % Data fit
    'k', 'LineWidth',2)         % Set line width to 2
hold off
set(gca, 'FontSize', 16)        % Increase font for printing
axis([0, 5, -50, 50])           % Set axis
title('Microphone 5')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_5*10)/10),'s'])


subplot(3,2,6)                  % Plot mic 6
plot(T, SPL6_B(1:Num_samples),...  % Ansys SPL
    'Color',[.5 .5 .5])  
hold on                         % Hold to overlay
plot(T,K6*X, ...                % Data fit
    'k', 'LineWidth',2)         % Set line width to 2
hold off
set(gca, 'FontSize', 16)        % Increase font for printing
axis([0, 5, -50, 50])           % Set axis
title('Microphone 6')
xlabel('Time (s)')
ylabel('SPL dB re 20 \mu Pa')
grid
% Write the T60 on the plot
text(3,10,['T60 = ',num2str(round(T60_Ansys_6*10)/10),'s'])

print -depsc 'reverberation_times_wb.eps'




%% Tabulate scalar parameters
disp('_____________________________________')
disp('Comparison of parameters in Workbench')
disp(['Parameter        ','Matlab   ','Ansys    ','% Diff'])
disp(['T60, Harmonic    ',num2str(T60),'  ',num2str(T60_ss),'  ', ...
    num2str(abs(T60_ss-T60)/T60*100)  ])
disp(['T60, Transient   ',num2str(T60),'  ',num2str(T60_Ansys_mean),'  ', ...
    num2str(abs(T60_Ansys_mean-T60)/T60*100)  ])

disp(['Alpha, Harmonic  ',num2str(Alpha_1),'  ',num2str(Alpha_ss),'  ', ...
    num2str(abs(Alpha_1 - Alpha_ss)/Alpha_1*100)])
disp(['Alpha, Transient ',num2str(Alpha_1),'  ',num2str(Alpha_Ansys_mean),'  ', ...
    num2str(abs(Alpha_1 - Alpha_Ansys_mean)/Alpha_1*100)])



