%% Radiation impedance of a baffled circular piston in an infinite baffle
% This script will plot the non-dimensional impedance of a baffle circular
% piston versus the frequency parameter 2ka
%
% References for this are:
%
% * Lawrence E. Kinsler, Austin R. Frey, Alan B. Coppens, James V. Sanders,
%   (1999), "Fundamentals of Acoustics", 4th edition, Wiley, .
% * Frank Fahy (1994), "Sound and structural vibration: 
%   radiation transmission and response", Academic Press , 
%   ISBN 012-247671-9, page 119-121, Figure 61.
% * David A. Bies and Colin H. Hansen (1997), 
%   "Engineering Noise control: theory and practice", E&F Spon,
%   ISBN 0-419-20430-X, pages 149-151.
% * Philip M. Morse AND K. Uno Ingard (1986), "Theoretical Acoustics", 
%   Princeton University Press, Princton, New Jersey, USA, 
%   ISBN: 0691024014.
%
% The goal is to generate Figure 7.5.2, p187 in Kinsler et al. (1999)
%
% Carl Howard, 27 November 2012
%
% Filename: |baffled_piston.m|

%% Define properties of air
c_speed_air=344;        % [m/s] speed of sound
rho_dens_air=1.21;      % [kg/m3] density of air

%% Define dimensions of piston
a_piston_radius=0.1;      % [m] radius of the piston


%% Define the analysis frequency range
% Figure 7.5.2, p187 Kinsler et al. has an x-axis
% corresponding to $x=2ka=0 \cdots 14$
x_axis=[0.1:0.1:14];    

% Alternatively can use this
%x_axis=logspace(-2,1,1000)                        

%%
% From this definition it is possible to back calculate 
% the frequency range of interest
k_wavenum = x_axis / (2*a_piston_radius);   % wavenumber
omega = k_wavenum*c_speed_air;              % [rad /s ] frequency

%% Calculate the Real and Imaginary parts of the impedance
% See Kinsler et al. (1999), p186 Eq. (7.5.11).
%
% [An alternative reference is: Morse and Ingard p384, Eq. (7.4.31)]
%
% The radiation impedance is given by
%
% $$ Z_r = \frac{F_{\omega}}{u_{\omega}} = \rho c (\pi a^2)
%                   [R_1(2ka)  -j X_1(2ka)] $$
%
% where the _REAL_ part of the impedance is 
% ( Kinsler et al. (1999) Eq. (7.5.12) )
%
% $$ R_1(x) = 1 - \frac{2 J_{1}(x)}{x} $$,
%
% the _IMAGINARY_ part of the impedance is
% ( Kinsler et al. (1999) Eq. (7.5.12) )
%
% $$ X_1(x) = \frac{2 H_1(x)}{x} $$
%
% $$ X_1(x)= \frac{4}{\pi} 
%   (
%       \frac{x}{3} 
%       - \frac{x^3}{3^2 \times 5} 
%       + \frac{x^5}{3^2 \times 5^2 \times 7} - \cdots
%   ) $$
%
% A thorough definition is in Morse and Ingard:
%
% $$ X_1 = \frac{4}{\pi} \int_{0}^{\pi/2} \sin ( \gamma ~ \cos \alpha)
% \sin^2 \alpha ~d \alpha $$
%
% where
% $\gamma = 2 k a $. Hence,  $\gamma=x=$ |x_axis|


%%
% This is the _REAL_ part of the impedance.
% Refer to:
%
% * Fahy p120
% * Kinsler et al. p186, Eq. (7.5.12)
% * Morse and Ingard, p384, Eq. (7.4.31)
%
R_1 = 1 - 2*besselj(1,x_axis)./x_axis;

%%
% This is the _IMAGINARY_ part of the impedance.
% Refer to:
%
% * Fahy p120
% * Kinsler et al. p186, Eq. (7.5.12)
% * Morse and Ingard, p384, Eq. (7.4.31)
%
% Uses the Struve function for the imaginary impedance, which is not
% an inbuilt Matlab function. An alternative was found on the internet.
X_1 = 2*struve(1,x_axis)./x_axis;

%% Plot the Impedance vs Frequency Parameter (2ka)
p1=plot(x_axis,R_1,x_axis,X_1,'--');
%p1=loglog(x_axis,R_1,x_axis,X_1,'--');
set(gca,'Fontsize',16)
set(p1(:),'Linewidth',2)
title 'Impedance of a baffled circular piston'
xlabel 'Frequency Parameter (2ka)'
ylabel 'Non-dimensional Impedance'
l1=legend('Real','Imaginary',0);
axis ([0 max(x_axis) 0 1.2]);
grid

%% Calculate the Radiated Acoustic Power
% The radiated acoustic power from the piston is given by 
%
% * Kinsler et al. (1999) Eq. (7.5.16), p187 (high frequency limit)
% * Bies and Hansen (2009) Eq. (5.111b), p213
%
% $$ W = \frac{1}{2} \, \rho_0 c R_1 (\pi a^2) U^2 $$
%
% where $R_1$ is the real part of the impedance in Bies and Hansen and 
% Kinsler et al., and is called $\theta_0$ in Morse and Ingard p384.

%%
% Define the analysis frequency range 
% The analysis range for Fig 7.5.2 is $ka=0 \cdots 14$. 
% Previously the values of $\omega$ were calculated, so the frequency is
% $f=\omega/(2 \pi)$
freq=omega/(2*pi);

%%
% If we assume that the displacement was 1 micron, 
% then the velocity can be calculated as
%
% $$ U=(1 \times 10^{-6}) \times j \omega $$
%
U_velocity = 1e-6 * j*omega;

% The radiated acoustic power is then
power = (1/2)* rho_dens_air * c_speed_air *   ...
        (pi * a_piston_radius^2) * R_1 .* abs(U_velocity).^2;



%% Plot radiated acoustic power vs frequency
figure
p2=plot(x_axis,10*log10(power/1e-12));
set(gca,'Fontsize',16)
set(p2(:),'Linewidth',2)
title 'Acoustic power radiated by a baffled circular piston'
xlabel 'Frequency Parameter (2ka)'
ylabel 'Acoustic Power [dB re 10^{-12}W]'
grid

return;
