%% Scattered Intensity From A Plane Wave Striking A Cylinder
% This script will calculate and plot the
%
% * incident sound pressure level of a plane wave in the absence of a
%   cylinder 
% * scattered pressure and 
% * scattered intensity
%
% from a plane wave striking an infinitely long, rigid walled cylinder.
%
% Carl Howard 7 Feb 2013

%% Introduction
% The mathematical model described in Morse and Ingard (1986) is an 
% acoustic plane wave that originates from the -x axis and propagates 
% to the +x axis (i.e. travelling from left to right). 
% Hence in the polar / directivity plots, the plane wave
% originates from 180 degrees and travels radially inwards to the centre 
% and then outwards along the 0 degrees radial.
%
% The plane wave strikes a rigid walled infinitely long cylinder,
% of radius $a$ centred at the origin, at an angle normal to the axis of
% the cylinder.
%
% References:
%
% * Philip M. Morse AND K. Uno Ingard (1986), "Theoretical Acoustics", 
%   Princeton University Press, Princeton, New Jersey, USA, 
%   ISBN: 0691024014.
% * Philip M. Morse (1981), "Vibration and Sound", Second edition
%   Acoustical Society of America, Woodbury, NY, USA, 
%   ISBN 0883188767.
% * Philip M. Morse AND Herman Feshbach (1953), 
%   "Methods of theoretical physics", volume 2,
%   McGraw-Hill book company, inc,  New York, USA,
%   ISBN 0976202115.
% * James J. Faran Jr. (1951)
%   "Sound scattering by solid cylinders and spheres",
%   The Journal of the Acoustical Society of America, vol. 23, n 4, July,
%   p405-418, <http://link.aip.org/link/?JAS/23/405/1>, 
%   DOI: 10.1121/1.1906780.
%
% This script can be used to generate figures to compare with:
%
% * Fig 8.1, p402, Morse and Ingard (1986) and
% * Figs 9 and 13 Faran (1951).  
%

%% Parameters of the cylinder
% Radius of the cylinder $a$
a_radius =  1.0;        % [m]

%% Parameters of the fluid
% speed of sound $c_0$
c_speed_sound   =   343.24;         % [m/s]

%%%
% Density of the fluid $\rho_0$
rho_density     =   1.2041;         % [kg/m^3]

%% Parameters of the analysis
% Wavelength $\lambda$
%
% Fig 8.1, p402 Morse and Ingard shows the scattered intensity at values of
%
% * $\lambda=2 \pi a$
% * $\lambda=\frac{2}{3} \pi a$
% * $\lambda=\frac{2}{5} \pi a$

%lambda_wavelength = 2*pi*a_radius;    % [m]
%lambda_wavelength = 2/3*pi*a_radius;    % [m]
lambda_wavelength = 2/5*pi*a_radius;    % [m]

%%%
% Other useful terms are
%
% Wavenumber $k=\omega/c_0=2\pi f /c = 2\pi / \lambda$
%
% [p401 Morse and Ingard]
k_wavenum = 2 * pi / lambda_wavelength;

%%%
% Another parameter that is used frequently in the analysis is $ka$
ka  =   k_wavenum * a_radius;

%%%
% Define the angles where results will be calculate for 
% polar / directivity plots.
% In Morse and Ingard they use the symbol $\phi$ for angle.
%
% We'll define $\phi=-\pi \cdots \pi$ because the script to plot
% directivity prefers -180 deg to 180 deg, but it should work if
% this angle is changed to your preference.
phi_angle = [-pi:0.01:pi];  % [radians]

%%%
% Radial distance where the pressure and intensity will be evaluated $r$
r_radius = 6.5;             % [m]

%% Parameters of the plane wave
% The amplitude of plane wave is [p401 Morse and Ingard]
%
% $$ P_0 = \sqrt{\rho_{0} c_{0} I } $$
%
% where the plane wave intensity is $I$.
%
% In Ansys, we define the amplitude of the pressure of the plane wave $P_0$
% and do not use the Intensity. So instead, lets define the plane wave
% pressure amplitude and calculate the corresponding intensity.

P_0 = 1.0;          % [Pa] pressure amplitude of plane wave
I_plane_wave_intensity = P_0^2 / (rho_density * c_speed_sound);

% Normally one would define the intensity and calculate the corresponding
% pressure amplitude.
%I_plane_wave_intensity =    1.0;
%P_0 =   sqrt(rho_density * c_speed_sound * I_plane_wave_intensity);

%%% 
% Number of terms to approximate the cylindrical waves $m=0 \cdots \infty$,
% and is approximated as $m=0 \cdots$|m_max_index|.
%
% Change the value of |m_max_index| as needed. 
% Check to make sure that the plot for the incident plane wave 
% in the absence of the cylinder has a _constant_ sound pressure level 
% on the arc of constant radius. 
% The SPL should be the same throughout the acoustic domain, 
% for all angles and radii.
m_index_incident_max = 500;               % change this value as required
m_index_incident = [0:m_index_incident_max];      % Must keep m_index(1)=0;

%%%
% The number of terms needed for the scattered pressure wave does not need
% to be the same as the incident wave. If the same number _is_ used, then
% NaN problems will occur when trying to evaluate the scattered pressure at
% large distances $(kr \gg 1)$.
m_index_scattered_max = 20;             % change this value as required
m_index_scattered = [0:m_index_scattered_max];    % Must keep m_index(1)=0;


%% Calculate the incident plane wave in terms of cylindrical waves
% This is not used in the solution, but is calculated and plotted for
% verification purposes. 
%
% The plane wave can be written as [p401, Eq. (8.1.1) Morse and Ingard]
%
% $$
%   p_p = P_0 
%         \left[
%           J_{0}(kr) + 
%           2 \sum_{m=1}^{\infty} i^{m} \cos(m \phi) \, J_{m}(kr)
%         \right] \,
%           e^{-2 \pi i \nu t}
% $$
%
% Note that the plane wave is calculated at a constant radius |r_radius|
% over a range of angles |phi_angle|, so we are calculating the incident
% pressure on an *arc* ... This is _not_ the usual way a comparison 
% would be made, as normallly one would want to plot 
% the incident sound pressure level at |phi_angle=0| and 
% the radius would be varied.

p_plane_wave=zeros(size(phi_angle));
J_0_kr = besselj(0,k_wavenum*r_radius);
J_m_kr = besselj(m_index_incident,k_wavenum*r_radius);
for nn=1:length(phi_angle)
    p_plane_wave(nn) = P_0 *                                        ... 
                    (                                               ...
                    J_0_kr +                                        ...
                    2*sum(                                          ...
                            1i.^(m_index_incident(2:end) ) .*                ...
                            cos(m_index_incident(2:end)*phi_angle(nn)) .*    ...
                            J_m_kr(2:end)                           ...
                          )                                         ...
                    );
end;



%% Plot the SPL of the incident plane wave
% Read the note from above -- this is the pressure on an arc at constant
% radius |r_radius| where the angle is varied.
%
% For a plane wave propagating in the absence of the cylinder, 
% the sound pressure level should be _constant_ throughout the domain, 
% for all angles and radii.
%
% If the plot does not show a (nearly) constant level (i.e. +/- 0.1 dB)
% then there is a problem with the calculations. Consider increasing the
% value of |m_max_index| and re-calculate this result.
%
% Calculate the sound pressure level as 
%
% $$
%   L_{p} = 20 \log_{10} 
%               \left[
%                   \frac{|p_{p}|}{\sqrt{2} \times 20 \times 10^{-6} }
%               \right]
% $$
p_plane_wave_dB = 20*log10(  abs(p_plane_wave)/sqrt(2)/20e-6 ); 

% Plot the SPL for the incident plane wave in the absence of the cylinder
figure
set(gca,'fontsize',16);
p1h=plot(phi_angle*180/pi,p_plane_wave_dB);
title(['Incident Plane Wave Measured on an Arc of radius=',  ...
        num2str(r_radius),'m']);
xlabel('Angle \phi [degrees]');
ylabel('SPL [dB RMS re 20\muPa]');
set(p1h,'linewidth',3);

%% Calculate the phase angle 
% The phase angle $\gamma_m$ determines the behavior of the scattered wave 
% which is written in [Morse p348, Eq. (29.2)] and 
% [ p401, Eq. 8.1.2 Morse and Ingard ] as
%
% $$
%   \tan \gamma_m =  
%        \frac{J_{m-1}(ka) - J_{m+1}(ka)}{
%              N_{m+1}(ka) - N_{m-1}(ka) }
% $$
%
% where 
%
% $J_{m}$ is Bessel function of 1st kind of order $m$, 
%           which is the |besselj| function in Matlab
%
% $N_{m}$ is the Neumann function, which is the same as the 
%           Bessel function of 2nd kind of order $m$, 
%           which is the |bessely| function in Matlab
%
% Tables of $\gamma_m$ are listed in
% [ p895, Table III Morse and Ingard ]

gamma_m = atan2( ( besselj(m_index_scattered-1,(ka)) -               ...
                    besselj(m_index_scattered+1,(ka)) )     ,       ...
                 ( bessely(m_index_scattered+1,(ka)) -               ...
                    bessely(m_index_scattered-1,(ka)) )             ...
               );

           
%%%           
% This set of numbers will approach zero as $m$ tends to large values, as 
% $N_{m}(ka) \rightarrow \infty$ as $m \rightarrow \infty$, and
% $J_{m}(ka) \rightarrow 0$ as $m \rightarrow \infty$.
%
% Hence we can set the elements to zero that are NaN
[aa]=isnan(gamma_m);
[aa,bb]=find(aa==1);
gamma_m(bb)=0;

%%%
% The special case is $m=0$ where [p401, Eq. 8.1.2 Morse and Ingard]
%
% $$
%   \tan \gamma_{0} = - \frac{ J_{1} (ka)}{ N_{1}(ka)}
% $$
%
% The values of $\gamma_m$  are listed in 
% [ Table V, p897 Morse and Ingard].
% For example, if $ka=3$, $\gamma_0 = 133.76$ deg $= 2.3346$ radians 
% in Morse and Ingard, but in Matlab, |gamma_0= -0.8071|
% The difference is:  pi - 0.8071 = 2.3346
%
% If you look at the equation (8.12) p401, 
% $\gamma_0 = \textrm{atan} (-J_1(ka) / N_1(ka) )$.
% These values can be found on p895, Morse and Ingard as
%   $J_1(3) = 0.3391, N_1(3)=0.3247$  
% which is the same as evaluated in Matlab.
%
%   |besselj(1,3)|=0.3391
%
%   |bessely(1,3)|=0.3247
%
% so |atan(- besselj(1,3)/ bessely(1,3)) = -0.8071|
% but |atan2(besselj(1,3), -bessely(1,3)) =  2.3345|
%
% Hence use the |atan2| expression.

gamma_m(1) = atan2( besselj(1,(ka)) , -bessely(1,(ka))  );
 
%% Calculate the scattered pressure
% The scattered pressure of the outgoing wave is [p401]
%
% $$ p_{s} = \sum_{m=1}^{\infty} A_{m} \cos( m \phi)
%               [ J_{m}(kr) + i N_{m} (kr) ]
%               \, e^{-2 \pi i \nu t}
% $$
%
% where [from p401, Eq (8.1.2) Morse and Ingard]
%
% $$ A_m = - \epsilon_{m} P_{0} i^{m+1} \, 
%              e^{-i \gamma_{m}} \sin(\gamma_m)
% $$ 
%
% The value of $\epsilon_m$ is [ p402 Morse and Ingard ]
%
% $$ \epsilon_{0} = 1 $$ for $m=0$
%
% $$ \epsilon_{m} = 2 $$ for $m>0$  
%
epsilon_m_scattered       = 2*ones(size(m_index_scattered));    
epsilon_m_scattered(1)    = 1;                        


A_m = - P_0 * epsilon_m_scattered .* (1i).^(m_index_scattered+1) .*  ...
        exp(-1i*gamma_m) .* sin(gamma_m);

%%%
% The expression: $[ J_{m}(kr) + i N_{m} (kr) ]$
% is actually the Hankel function $H_{m}^{1} (kr)$ 
% [ Morse and Feshback, p1371, Eq. (11.2.22) ]
% which can be evaluated in Matlab as
% | besselh(m,1,kr) |
%
% Define 
%
% |constant_term| = $[ J_{m}(kr) + i N_{m} (kr) ] = H_{m}(kr)$
%
% where $H_{m}(kr)$ is the Hankel function

constant_term = besselh(m_index_scattered,1,k_wavenum*r_radius);

% This is the "long hand" expression which you can use to verify that it
% evaluate to the Hankel function
% constant_term = (   besselj(m_index, k_wavenum*r_radius) +  ...
%                     1i*bessely(m_index,k_wavenum*r_radius)  ...
%                 );

%%%
% *WARNING*
%
% The formula for scattered pressure $p_s$ in Morse and Ingard has 
% $\sum_{m=1}^{\infty}$. However this will give incorrect results:
%
% * does not match Ansys predictions and 
% * does not match predictions using Junger's theory 
%   [Junger and Feit (1993)].
%
% It is necessary to change the summation to $\sum_{m=0}^{\infty}$. 
%
% For your own verification, you can change the Matlab code to the 
% expression in Morse and Ingard where the summation is over 
% $m=1 \cdots \infty$ and skips $m=0$, so the index range 
% will be |m_index(2:end)|


%%%
% Use a |for| loop to calculate the scattered pressure at each angle of
% $\phi$.
%
p_scattered = zeros(size(phi_angle));          
for nn=1:length(phi_angle)

% This code will give inaccurate results, see warning note above.
%     p_scattered(nn) = sum(                                            ...
%                             A_m(2:end).*                              ...
%                             cos(m_index_scattered(2:end)*phi_angle(nn)).*       ...
%                             constant_term(2:end)                      ...
%                           );

    p_scattered(nn) = sum(                                          ...
                            A_m.*                                   ...
                            cos(m_index_scattered*phi_angle(nn)).*  ...
                            constant_term                           ...
                          );                      

                          
end;

%%%
% Calculate the Sound Pressure Level (SPL), which uses the 
% Root-Mean-Square (RMS) value of the scattered pressure 
% |p_scattered|, so it is necessary to divide by $\sqrt{2}$.
p_scattered_dB = 20*log10(  abs(p_scattered)/sqrt(2)/20e-6 );


%% Calculate the acoustic particle velocity in the RADIAL direction
% This section will calculate the acoustic particle velocity in the
% _radial_ direction. Note that if predictions are made using Ansys, which
% will by default calculate the velocity along the $x$ and $y$ axes, it is
% necessary to transform the results to determine the velocity in the
% radial direction.
%
% The acoustic particle velocity is [p401, Morse and Ingard]
%
% $$ 
%     u_{sr} = \frac{1}{\rho c}
%         \Bigl\{
%             i A_0 [ J_1 (kr) + i N_1 (kr) ]   
% $$ 
%
% $$        \quad
%             + \frac{i}{2} \sum_{m=1}^{\infty} A_m \cos (m \phi) 
%               [ J_{m+1} (kr) - J_{m-1} (kr)    
%                           + i N_{m+1} (kr) - i N_{m-1} (kr)] 
%         \Bigr\}
%         e^{-2 \pi i \nu t}
% $$

constant_term_1 = 1i*A_m(1)*besselh(1,1,k_wavenum*r_radius);

% Using the relationship for the Hankel function
constant_term_2 = besselh(m_index_scattered+1,1,k_wavenum*r_radius)       ...
                    - besselh(m_index_scattered-1,1,k_wavenum*r_radius);

% "Long Hand" version
% constant_term_2 = besselj(m_index+1,k_wavenum*r_radius)       ...
%                - besselj(m_index-1,k_wavenum*r_radius)      ...
%                + 1i*bessely(m_index+1,k_wavenum*r_radius)   ...
%                - 1i*bessely(m_index-1,k_wavenum*r_radius) ;
                
             
u_sr = zeros(size(phi_angle));
for nn=1:length(phi_angle)
    u_sr(nn) = ( 1 / (rho_density * c_speed_sound) ) *          ...                
               (                                                ...
                constant_term_1 +                     ...                
                1i/2* sum(                                      ...
                        A_m(2:end).*                            ...
                        cos(m_index_scattered(2:end)*phi_angle(nn)) .*    ...
                        constant_term_2(2:end)                  ...
                          )                                     ...
                );
end;

%% Calculate the scattered pressure at large distances from the cylinder
% See p402, Morse and Ingard
%
% The scattered pressure at large distances from the cylinder is 
%
% $$
%   p_{s} = - \, \sqrt{ \frac{  2 \rho_{0} c_{0} I a}{\pi r} } \,
%           \psi_{s} (\phi) e^{i k (r - ct)}
% $$
%
% where
%
% $\psi_{s} (\phi)$ is defined as [p402, Morse and Ingard]
%
% $$
%   \psi_{s} (\phi) = \frac{1}{\sqrt{ka}}
%           \sum_{m=0}^{\infty} \epsilon_{m} 
%               \sin (\gamma_{m}) \, e^{\, -i \gamma_{m}} \, 
%               \cos (m \phi)
% $$
%
% Notes:
%
% * The exponential expression in Morse and Ingard was meant to be 
% written as
%
% $$  e^{-i \gamma_{m}} $$
%
% Rather than as it appears in the textbook on [p402]
%
% $$  e^{-i \gamma {m}} $$
%

%%%
% Calculate the value for $\psi_{s} (\phi)$ using a 
% |for| loop to evaluate it at each angle in the vector 
% $\phi$.
%
% Pre-calculate the constant term 
% $\epsilon_{m} \sin (\gamma_{m}) \, e^{\, -i \gamma_m}$
%
constant_term_2 = epsilon_m_scattered .* sin(gamma_m) .* exp(-1i*gamma_m);

%%%
% Create an empty array that will store all the results for 
% $\psi_{s} (\phi)$
psi_s_phi = zeros(size(phi_angle));

% Now the for loop
for nn=1:length(phi_angle)
    psi_s_phi(nn) = (1/sqrt(ka))*   ...
                    sum(constant_term_2.*   ...
                        cos(m_index_scattered*  ...
                        phi_angle(nn) ) );
end;    

%%%
% Calculate the scattered pressure at large distances from the cylinder
p_scattered_large_dist = -sqrt(        ...
                        (2*rho_density*c_speed_sound*   ...
                        I_plane_wave_intensity*a_radius     ) / ...
                        (pi*r_radius)   ...
                    ) *             ...
                    psi_s_phi*  ...
                    exp(1i*k_wavenum*r_radius);

% Calculate the SPL, which uses the RMS value of p_scattered_large_dist, 
% so it is necessary to divide by sqrt(2).
p_scattered_large_dist_dB = 20*log10(   ...
                            abs(p_scattered_large_dist)/sqrt(2)/20e-6 );
                
 
%% Plot the scattered pressure

figure
subplot(2,1,1);
set(gca,'fontsize',16);
p1h=plot(   phi_angle*180/pi,real(p_scattered),'-',       ...
            phi_angle*180/pi,real(p_scattered_large_dist),'--'  ...
         );
set(p1h,'linewidth',3);
ylabel('Real Pressure [Pa]');
grid
title(['Scattered Pressure at \lambda=',   ...
        num2str(lambda_wavelength),'m']);
l2h=legend('Real P: Small Dist. Eq.',   ...
            'Real P: Large Dist. Eq.',  ...
            'location','southeast');
    
subplot(2,1,2);
set(gca,'fontsize',16);
p1h=plot(   phi_angle*180/pi,imag(p_scattered),'-',       ...
            phi_angle*180/pi,imag(p_scattered_large_dist),'--'  ...
         );
xlabel('Angle \phi [degrees]');
ylabel('Imag. Pressure [Pa]');
grid
set(p1h,'linewidth',3);
l2h=legend( 'Imag P: Small Dist. Eq.',   ...
            'Imag P: Large Dist. Eq.',  ...            
            'location','southeast');

figure
set(gca,'fontsize',16);
p2h=plot(phi_angle*180/pi,p_scattered_dB,   ...
        phi_angle*180/pi,p_scattered_large_dist_dB,'--');
set(p2h,'linewidth',3);
title(['Scattered Pressure at \lambda=',    ...
        num2str(lambda_wavelength),'m']);
xlabel('Angle \phi [degrees]');
ylabel('SPL [dB RMS re 20\muPa]')
l2h=legend('Small Dist. Eq.',   ...
            'Large Dist. Eq.',  ...
            'location','southeast');
grid

%p3h=Dirplot(phi_angle*180/pi,p_scattered_dB);
%polar(phi_angle,abs(p_scattered));


%% Calculate the scattered radial velocity (at large distances)
% The corresponding scattered radial velocity at large distances 
% from the cylinder is given by [p402, Morse and Ingard]
%
% $$
%   u_{sr} = \frac{p_s}{\rho_{0} c_{0}}
% $$
u_sr_large_dist = p_scattered_large_dist/(rho_density * c_speed_sound);


%% Plot the real and imaginary parts of scattered radial velocity
figure
subplot(2,1,1);
set(gca,'fontsize',16);
p4h=plot(phi_angle*180/pi,real(u_sr),'-',     ...
        phi_angle*180/pi,real(u_sr_large_dist),'--');
title({'Scattered Radial','Acoustic Particle Velocity [m/s]'});
xlabel('Angle \phi [degrees]');
ylabel('Real Velocity [m/s]');
set(p4h,'linewidth',3);
l2h=legend('Small Dist. Eq.',   ...
            'Large Dist. Eq.',  ...
            'location','southeast');
grid

subplot(2,1,2);
set(gca,'fontsize',16);
p4Bh=plot(phi_angle*180/pi,imag(u_sr),'-',     ...
        phi_angle*180/pi,imag(u_sr_large_dist),'--');
title({'Scattered Radial','Acoustic Particle Velocity [m/s]'});
xlabel('Angle \phi [degrees]');
ylabel('Imag Velocity [m/s]');
set(p4Bh,'linewidth',3);
l4Bh=legend('Small Dist. Eq.',   ...
            'Large Dist. Eq.',  ...
            'location','southeast');
grid


%% Calculate the scattered intensity
% Although not shown in Morse and Ingard, the 
% scattered (real) intensity can be
% calculated as [Bies and Hansen (2009), Eq. 1.72, p35]
%
% $$
%   I_{s} = \frac{1}{2}\,\textrm{Re} ( p_{s} u_{s}^{*})
% $$
I_scattered = 1/2*real(p_scattered.*conj(u_sr));

%% Calculate the scattered intensity at large distances from cylinder
% This section will calculate the intensity of the scattered wave
% and can be used to plot the polar diagrams from
% Morse and Ingard, p402, Fig 8.1
%
% The intensity of the scattered part at a point $(r,\phi)$ 
% at a _large_ distance from the cylinder $(kr >> 1)$ is
% [Eq. (8.1.3), p402, Morse and Ingard]
%
% $$
%   \Upsilon_{s} \approx \frac{I a}{\pi r} \, |\psi_{s}(\phi)|^2
% $$
%
% where
%
% $$
%   |\psi_{s}(\phi)|^2 = \frac{1}{k a}
%           \sum_{m,n=0}^{\infty} \epsilon_{m} \epsilon_{n}
%                   \sin \gamma_{m} \sin \gamma_{n}
%                   \cos ( \gamma_{m} - \gamma_{n} )
%                   \cos (m \phi) \cos (n \phi)
% $$
%
% It can be shown that the long expression for
% $|\psi_{s}(\phi)|^2$ beneath Eq. (8.1.13) evaluates to the same value as
% |abs(psi_s_phi).^2|.  
% 

% This is the long hand version where the long equation beneath 
% Eq. (8.1.13) is evaluated.
%
% Precalculate the constant term
% temp=zeros(length(m_index),length(m_index));
% for mm=1:length(m_index)
%     for nn=1:length(m_index)
%         temp(mm,nn) =  epsilon_m(mm)*epsilon_m(nn)*         ...
%                 sin(gamma_m(mm))*sin(gamma_m(nn)) *         ...
%                 cos(gamma_m(mm) - gamma_m(nn)) ;
%     end;
% end;
% 
% psi_s_phi_squared = zeros(size(phi_angle));
% for ii=1:length(phi_angle)
%     psi_s_phi_squared(ii) = 1/(ka) *                    ...
%                 sum(                                    ...
%                     sum(    temp.*                      ...
%                 ( repmat(cos(m_index.'*phi_angle(ii)),1,length(m_index)) ) .*   ...
%                 ( repmat(cos(m_index*phi_angle(ii)),length(m_index),1) )    ... 
%                         )                                               ...
%                    );
% end;

% Calculate the Upsilon_s expression which is the scattered intensity. 
% To make this more readable we will call I_scattered = Upsilon_s
% I_scattered_large_dist = (I_plane_wave_intensity*a_radius)/   ...
%                            (pi*r_radius) *   psi_s_phi_squared;

I_scattered_large_dist = (I_plane_wave_intensity*a_radius)/     ...
                            (pi*r_radius) * abs(psi_s_phi).^2;
            

%% Plot the scattered intensity
figure;
set(gca,'fontsize',16)
p6h=plot(phi_angle*180/pi,I_scattered,  ...
        phi_angle*180/pi,I_scattered_large_dist,'--');
title({'Scattered Intensity',   ...
            ['at \lambda=',num2str(lambda_wavelength),'m']});
set(p6h,'linewidth',3);
l6h=legend('Short Dist. Eq.','Large Dist. Eq.',0);
grid



% Polar plot
% The polar graphs in Matlab are not easy to scale. It is necessary to
% trick Matlab to display the correct scale by initially plotting a dummy
% polar graph with the desired scale.
figh=figure;
gca1=subplot(1,2,1);
set(gca1,'fontsize',16);
linestyles = cellstr(char('-',':','-.','--','-',':','-.',   ...
                            '--','-',':','-',':',           ...
                            '-.','--','-',':','-.','--','-',':','-.'));
default_line_colors=get(gca1,'colororder');

p7h=polar(0,1,'w-');
hold on

% Find the largest value from I_scattered and I_scattered_large_dist
% and use this to normalise the results
I_scattered_max = max(I_scattered);
I_scattered_large_dist_max=max(I_scattered_large_dist);
I_scat_norm = max(I_scattered_max,I_scattered_large_dist_max);

p8h=polar(phi_angle,I_scattered/I_scat_norm,'-');
set(p8h,'linewidth',3);
hold on 
p9h=polar(phi_angle,I_scattered_large_dist/I_scat_norm,'--');
set(p9h(1),'Color', default_line_colors(1,:));
set(p9h,'linewidth',3);

% title({'Normalised Scattered Intensity',   ...
%             ['at \lambda=',num2str(lambda_wavelength),'m']});

title({'Normalised Scattered Intensity',   ...
             ['at \lambda=(2/3)\pia']});


set(p9h(1),'linewidth',3);
set(p9h(1),'Color', default_line_colors(2,:));
set(p9h(1),'linestyle',linestyles{4});
hold on

set(figh,'ActivePositionProperty','OuterPosition');

gca2=subplot(1,2,2);
% Find the location of these axes
pos_gca2=get(gca2,'position');

l9h=legend([p8h,p9h],{'Short Dist. Eq.','Large Dist. Eq.'}, ...
                'location','EastOutside');
pos_l9h=get(l9h,'position');

new_legend_pos=pos_l9h;
new_legend_pos(1)=pos_gca2(1);
new_legend_pos(2)=pos_gca2(2)+pos_gca2(4)/2-pos_l9h(4)/2;

set(l9h,'position',new_legend_pos);

delete(gca2);

            