%% Radiation from a Monopole
% This script will plot the theoretical sound pressure level radiated by a
% monopole versus angle.
%
% References 
%
% * David A. Bies and Colin H. Hansen (2009), 
%   "Engineering Noise Control: Theory and Practice", 
%   Fourth edition, Spon Press.
% * Daniel A. Russell, Joseph P. Titlow, and Ya-Juan Bemmen (1999),
%   "Acoustic monopoles, dipoles, and quadrupoles: 
%   An experiment revisited",
%   American Journal of Physics, 67 (8), August , p660-664.
%   <http://dx.doi.org/10.1119/1.19349>
%
% Carl Howard 10 January 2013

%% Theory
% The sound pressure from a monopole source is given by
% 
% $$
%   p_{\textrm{monopole}}  = \frac{j \rho_0 c_0 k S u}{4 \pi r} \, 
%                               e^{j(\omega t - kr)}
% $$
%    
% where 
%
% * $S$ is the surface area of the sphere of radius $a$ given by
% 
% $$    S = 4 \pi a^2   $$
%
% * $u$ is the velocity of the expanding and contracting 
% surface of the sphere, 
% * $\rho_0$ is the density of the acoustic fluid, 
% * $c_0$ is the speed of sound of the acoustic fluid,
% * $\omega=2 \pi f$ is the angular frequency, 
% * $k=\omega/c_0$ is the wavenumber, and 
% * $r$ is the distance from the centre of the sphere to the 
% measurement location.

%% Define the parameters of the monopole

%%% 
% Radius of the monopole sphere $a$
a_radius        = 0.01;         %[m]

%%%
% Velocity of the surface of the sphere $u$
u_velocity      = 1.0;          % [m/s]

%%%
% Surface area of monopole sphere
S_area          = 4*pi*a_radius^2;

%%%
% Volume velocity of monopole $Q = S \, u$
Q_volume_velocity = S_area * u_velocity;


%% Properties of the fluid
%

%%% 
% Density of the fluid $\rho$
rho_density     =   1.2041;     % [kg/m^3]

%%% 
% Speed of sound of the fluid $c$
c_speed_sound   =   343.024;    % [m/s]

%%% 
% Reference Sound Pressure $p_{\rm{ref}}$
p_ref           =   20e-6;      % [Pa]

%% Analysis configuration
% Harmonic frequency of analysis
freq            = 500;          % [Hz]
omega           = 2*pi*freq;    % [radians/sec]
k_wavenum       = omega/c_speed_sound; 

%%% 
% Angle over which the SPL is to be calculated $\theta$.
% For the |Dirplot| matlab function, the angle |theta| 
% has to be in degrees, and for a full circle 
% is between -180 to 180 degrees
theta           = [-180:1:180]; % [degrees]

%%% 
% Radius where SPL is to be evaluated
r_radius        = 4;            % [m]


%% Calculate the sound pressure of a monopole
% The sound pressure from a monopole source is given by
% 
% $$
%   p_{\textrm{monopole}}  = \frac{j \rho_0 c_0 k S u}{4 \pi r} \, 
%                               e^{j(\omega t - kr)}
% $$
%
p_monopole      = 0;
p_monopole      = abs(  ...
                    (1i *rho_density*c_speed_sound*    ...
                        k_wavenum* Q_volume_velocity)   /  ...
                    (4*pi*r_radius)         ...
                        );

                    
%%%
% The sound pressure level in decibels is given by
%
% $$ L_p = 20 \log_{10} 
%               \left[
%                   \frac{p_{\rm{monopole}}}{\sqrt{2} \, p_{\rm{ref}}}
%               \right]
% $$
%
SPL_monopole      = 20*log10(p_monopole/p_ref/sqrt(2));
SPL_monopole      = ones(size(theta))*SPL_monopole;  

%% Plot the SPL versus angle
p1h=Dirplot(theta,SPL_monopole,'',[80 0 8]);
set(p1h(1),'linewidth',3);
title('SPL vs Angle for a Monopole Source');

%% Calculate the sound power radiated from monopole
%
% The theory is from Bies and Hansen (2009), Eq. (5.12)
% The theoretical sound power radiated by a monopole is 
%
% $$
%    W_{\textrm{monopole}} =\frac{ Q_{\textrm{RMS}}^{2} k^2 \rho_0 c_0}{
%            4 \pi \, (1+k^2 a^2)}
% $$  
%
% where the RMS volume velocity of the monopole is 
% [Eq. (5.7) Bies and Hansen (2009)]
%
% $$
%    Q_{\textrm{RMS}} = 4\pi a^2 \, U_{0, \textrm{RMS}}
% $$
%
% and $U_{0, \textrm{RMS}}=U_{0} / \sqrt{2}$ is the 
% RMS amplitude of the velocity of the surface of the sphere.
%

Q_RMS = 4 * pi * a_radius^2 * u_velocity / sqrt(2);

W_monopole = Q_RMS^2 * k_wavenum^2 * rho_density * c_speed_sound / ...
                (4 * pi * (1+ k_wavenum^2 * a_radius^2) );

L_w_monopole = 10*log10(W_monopole / 1e-12)

%% Alternative equation for sound power
% You might also see the expression for sound radiated by a monopole
% written as [Eq. (4), Russell et al. (1999)]
%
% $$
%    W_{\textrm{monopole}}=\frac{Q^{2} k^2 \rho_0 c_0}{
%            8\pi}
% $$  
%
% where $Q$ is the amplitude of the volume velocity from the monopole given
% by
%
% $$
%    Q = 4\pi a^2 \, U_{0}
% $$
% 
% and $U_0$ is the amplitude of the velocity of the surface of the sphere.
%
% Although the expressions appear different, they give nearly identical
% results, as the terms $k^2 a^2$ in the denominator can be ignored for
% very small sources where the radius is much less than the wavelength
% (i.e. $a < < \lambda$) [p190, Bies and Hansen (2009)]. 
% The expressions for the volume velocity and the denominator 
% can be shown to be equivalent as
% 
% $$ 
%   \frac{ Q_{\textrm{RMS}}^{2} }{4 \pi} = 
%           \frac{ (4\pi a^2 \, U_{0, \textrm{RMS}})^{2} }{4\pi}  \\
%   {}  = \frac{ (4\pi a^2 \, U_{0} / \sqrt{2})^{2} }{4\pi}     \\
%   {}  = \frac{ (4\pi a^2 \, U_{0})^{2} }{8\pi}     \\
%   {}  = \frac{ Q^{2} }{8\pi}     \\
% $$
%
Q = 4 * pi * a_radius^2 * u_velocity;
W_monopole_2 = Q^2 * k_wavenum^2 * rho_density * c_speed_sound / ...
                (8 * pi);
L_w_monopole_2 = 10*log10(W_monopole_2 / 1e-12)

%%%
% and hence the results are nearly identical
disp(['L_w_monopole = ',num2str(L_w_monopole),' dB re 1e-12W']);
disp(['L_w_monopole_2 = ',num2str(L_w_monopole_2),' dB re 1e-12W']);
disp(['Difference = ',num2str(abs(L_w_monopole-L_w_monopole_2)),' dB']);
