%% On-Axis pressure from baffled circular piston
% This script is used to calculate the on-axis sound pressure  
% from a baffled circular piston
%
% References:
%
% * Lawrence E. Kinsler, Austin R. Frey, Alan B. Coppens, James V. Sanders,
%   "Fundamentals of Acoustics", 4th edition, Wiley, 1999.
%   See Section 7.4 Radiation from a plane circular piston
% * David A. Bies and Colin H. Hansen, "Engineering Noise control: theory
%   and practice", E&F SPON, 1997, ISBN 0-419-20430-X, pages 149-151.
%
% The goal of this script is to generate Figure 7.4.2, p181 in 
% Kinsler et al. (1999)
%
% Carl Howard, 27 November 2012
%

%% Define properties of air
c_speed_air=343;        % [m/s] speed of sound
rho_dens_air=1.21;      % [kg/m^3] density of air

%% Define properties of piston 
a_piston_radius=0.1;      % [m] radius of the piston

%% Define analysis frequency range
% The goal is to generate figure 7.4.2 , p181 in Kinsler et al.
% The parameters used are 
%
% $$ k a = 8 \pi  $$
%
% $$ (\omega / c) ~a = 8 \pi $$
%
% $$ (2 \pi f_{\rm max} / c) ~a = 8 \pi $$
%
% Hence, for a given |a_piston_radius|, the maximum analysis frequency is
%
% $$  f_{\rm max} = 8 c / (2 a) $$ [Hz]
%
freq = 8*c_speed_air / (2*a_piston_radius);      % [Hz] frequency 

%%
% The wavenumber $k$ is
%
% $$ k = 2 \pi f / c $$
%
k_wavenum = 2*pi*freq/c_speed_air;      % wavenumber

%%
% The |x_axis| range is $r/a=0 \cdots 8$ in
% Figure 7.4.2 in Kinsler et al.
r_radius = [0.001:0.001:8]*a_piston_radius;

% Alternatively the results can be calculated with a log spacing,
% which is useful if the x-axis on the graph is logarithmic.
% r_radius = logspace(-3,1,1000)*a_piston_radius;

% The maximum velocity of the piston is defined here as
U_max_vel = 1e-6*2*pi*freq;   % [m/s] maximum velocity of the piston

%%
% The complex pressure on axis is given by Eq (7.4.4) Kinsler et al.
%
% $$ p(r,0,t) = \rho_0 c U_0 
%                   \left\{
%                       1 - \exp[-j k ( \sqrt{r^2 + a^2} -r) ]
%                   \right\}
%                   e^{j(\omega t -kr)} $$
%
p_on_axis   =   rho_dens_air*c_speed_air*U_max_vel*    ...
        (           ...
            1-exp(-1i*k_wavenum* ...
                    (sqrt(r_radius.^2 + a_piston_radius.^2) - r_radius) ...
                 )  ...
        );

%%    
% The pressure amplitude is the absolute value of this expression    
p_on_axis = abs(p_on_axis);

%% Calculate where the pressure zeros (null) occur
% Pressure nulls occur at
%
% $$ r(n) = \frac{a^2 f^2 - n^2 c^2}{2 n c f} $$
%
% where $n=1$ to the lowest integer of 
%
% $$ n < \frac{a f}{c} $$
%
% In Fig 7.4.2 there are 4 pressure zeros. 
% Hence for $n=1 \cdots 4$
n = [1:floor(a_piston_radius*freq/c_speed_air)];
r_nulls = (a_piston_radius^2 * freq^2 - n.^2 * c_speed_air^2) ./   ...
            (2 * n * c_speed_air * freq);

% Create x and y vectors of the pressure nulls to plot
% If you want to plot vertical lines at the nulls use these expressions
%x_r_nulls = [r_nulls ; r_nulls] / a_piston_radius;
%y_r_nulls = [zeros(1,length(n)) ; ones(1,length(n))];
x_r_nulls = [r_nulls ] / a_piston_radius; 
y_r_nulls = zeros(size(x_r_nulls));


%% Plot of axial pressure amplitude 
% Re-produce figure 7.4.2 , p181 in Kinsler et al.
x_axis = r_radius/a_piston_radius;
y_axis = p_on_axis / (2*rho_dens_air*c_speed_air*U_max_vel);
% If you are plotting vertical lines at the pressure nulls use this line
%p1=plot(x_axis,y_axis,x_r_nulls,y_r_nulls,'r--');
p1=plot(x_axis,y_axis,x_r_nulls,y_r_nulls,'ro');
set(gca,'Fontsize',16)
set(p1(:),'Linewidth',2)
title 'Axial pressure of a baffled circular piston ka=8\pi'
xlabel 'Non-dimensional distance (r/a)'
ylabel 'Non-dimensional Pressure (P/2 \rho cU)'
axis ([0 max(x_axis) 0 max(y_axis)]);
legend('Theory: Pressure','Theory: Zeros','location','southeast') 
