%% Calculation of the transmission loss of a plate in an infinite baffle
% DESCRIPTIVE TEXT
function TL=TL_roussos;

%
% Reference:
%
% * Louis A Rossos,
% "Noise Transmission Loss of a rectangular plate in an infinite baffle",
% NASA technical paper 2398,
% March, 1985,
% Langley Research Centre,
% Hampton, Virginia
%
% The script will calculate the transmission loss of the panel described in
% the paper by Roussos and plot the results as shown in Figure 7 of the
% paper. The results are compared with the predictions for an infinite
% panel, and also the theory of Sewell.
%
%
% Coded by:
% Carl Howard
% 24 July 2006



%% Dimensions of the plate

% Dimensions in Roussos paper
plate.len_x = 0.38;     % [m]
plate.len_y = 0.15;     % [m]
plate.thick = 0.00081;  % [m]

% plate.len_x = 1.0;     %m
% plate.len_y = 1.5;     %m
% plate.thick = 0.0015;   %m



%%%
% Material properties of Aluminium
% These were not specified in the paper, just guessing
plate.density   = 2700;		%[kg/m^3] density
plate.E		    = 70e9;		%[Pa] Youngs modulus
plate.nu	    = 0.33;		%[no units] Poissons Ratio

% From Eq. 1.2 in Leissa (p1)
plate.D     =   plate.E * plate.thick^3 / (12 * (1-plate.nu^2) );

% Loss factor for plate
% These were not specified in the paper, just guessing
%plate.strloss     = 0.1;    
plate.damp_ratio  = 0.10;

%%%
% Mass per unit area of plate defined as $m_p$ in Roussos (1985)
plate.m_p =  plate.density*plate.thick;


% Example for Fig 7 in Roussos paper
theta_i = 60*pi/180;
phi_i = 0*pi/180;

% theta_i = 0*pi/180;     %[radians]
% phi_i = 0*pi/180;       %[radians]



% These were not specified in the paper, just guessing
fluid.rho = 1.21;		% [kg/m^3] density
fluid.c  = 343;         % [m/s] speed of sound

pressure_incident = 1;  % [Pa]


%% Location of the microphone
mic.loc.x = 0.0;
mic.loc.y = 0.0;
mic.loc.z = 1.0;

%% Location of accelerometer
% accel.loc.x = plate.len_x/2 + 0.13300;
% accel.loc.y = plate.len_y/2 +  0.45000E-01;
accel.loc.x = plate.len_x/2;
accel.loc.y = plate.len_y/2;

% corresponds to node
% 1823   0.13300     0.45000E-01  0.0000 



%% Convert the mic location from rectangular to spherical coordinates
% There is an inbuilt Matlab function to do the conversion.
% azimuth and elevation are angular displacements in radians measured 
% from the positive x-axis, and the x-y plane, respectively; 
% and r is the distance from the origin to a point.
[mic.loc.azimuth,mic.loc.elevation,mic.loc.r] =     ...
                            cart2sph(mic.loc.x,mic.loc.y,mic.loc.z);
% referring to Fuller, Fig 8.2 , p225
mic.loc.phi = mic.loc.azimuth;
mic.loc.theta = pi/2-mic.loc.elevation;



% The number of modes to include in the analysis (integer)
% These were not specified in the paper, just guessing
%number_of_modes_along_each_axis = 60;       % This is adequate for Roussos
number_of_modes_along_each_axis = 80;       % Use 80 for CQH paper
number_of_modes_in_analysis = (number_of_modes_along_each_axis).^2;


% Frequency range to analyse (in Hertz)
frequency_range = logspace(1,4.4,200); % [Hz] Use this for Roussos Paper
%frequency_range = [100:10:500];          % [Hz] Use this for CQH experiment


%% The theory from Sewell 
% described in: 
% Fahy and Gardonio, Sound and Structural Vibration, p299 Eq (5.57) 
% for a finite panel in an infinite baffle.
%
% Critical Frequency $\omega_c = c^2 (m/D)^{1/2}$
% Fahy and Gardonio p287, Eq 5.41
omega_c = fluid.c^2 * sqrt(plate.density*plate.thick/plate.D);

%%%
% Fahy and Gardonio p291, Eq 5.46
TL_0 = 20*log10(2*pi*frequency_range*plate.density*plate.thick*cos(0)/ ...
                    (2*fluid.rho*fluid.c));

%%%
% Fahy and Gardonio, p299 Eq (5.57)                
TL_sewell = TL_0 - 10*log10( log(   frequency_range*2*pi /     ...
                               fluid.c*sqrt(plate.len_x*plate.len_y))) ...
            +20*log10(1 - (frequency_range*2*pi ./  ...
                            (ones(size(frequency_range))*omega_c)).^2);

                        
%% Infinite panel theory  
% Reference is from 
%
% *  Beranek and Ver, Noise and Vibration Control Engineering,
%   p286, Eq (9.85)
%
% Note that the Roussos paper did not define which theory was used for the
% comparisons of the finite panel and infinite panel methods.
TL_beranek=10*log10(    ...
            ones(size(frequency_range)) + abs(  ...
                    (plate.density*plate.thick*frequency_range*2*pi)/   ...
                    (2*fluid.rho*fluid.c/cos(theta_i))  ...
                .*(     ones(size(frequency_range)) -    ...
                    (frequency_range*2*pi ./    ...
                        (ones(size(frequency_range))*omega_c)).^2   ...
                * (sin(theta_i)).^4 ) ).^2  ...
                );

        
        
%% Generate all the modal indices
%
all_modal_indicies=zeros((number_of_modes_along_each_axis).^2,3);
for ii=1:number_of_modes_along_each_axis
    for ff = 1:number_of_modes_along_each_axis
            all_modal_indicies((ii-1)*  ...
                (number_of_modes_along_each_axis)+(ff),:) = [ 0 ii ff ];
    end;
end;

%%%
% Overwrite the zeros 0 in the first column with the sequential list
% Remember that this list is not all that useful at the moment.
all_modal_indicies(:,1)=[1:((number_of_modes_along_each_axis).^2)].';

%%%
% The m and n values are the 2nd and 3rd columns
m_index = all_modal_indicies(:,2);
n_index = all_modal_indicies(:,3);


%==========================================================================
%% Calculate the resonance frequencies for the plate
%==========================================================================

% Resonance frequencies of a simply supported plate are:
% Equation 12
omega_mn = sqrt(    ...
                plate.D * pi^4 / (plate.density*plate.thick) *   ...
                ((m_index/plate.len_x).^2+(n_index/plate.len_y).^2 ).^2 ...
            );

        
%% Calculate the incident acoustic power 
% Equation 5
power_incident = pressure_incident.^2 *     ...
                plate.len_x * plate.len_y * cos(theta_i) /  ...
                (2 * fluid.rho * fluid.c);


%% Calculate the modal mass       
% The modal mass of a simply supported plate is 1/4 of the total mass of
% the plate. This is used to evaluate the structural kinetic energy.
modalmass = plate.m_p * plate.len_x * plate.len_y / 4;

%% Calculate the mode shapes at the location of the accelerometer
accel_plate_mode_shape = sin(m_index*pi*accel.loc.x/plate.len_x)    ...
                        .*sin(n_index*pi*accel.loc.y/plate.len_y);


%%%
% pre-allocated empty matrices
power_transmitted = zeros(length(frequency_range),1);
p_mn=zeros(length(m_index),length(frequency_range));
%W_mn=zeros(length(m_index),length(frequency_range));
mic_pressure=zeros(length(frequency_range),1);

accel_displ=zeros(length(frequency_range),1);

for ff=1:length(frequency_range);

    % Print the frequency on the screen
    fprintf('Frequency = %6.1f \n',frequency_range(ff));

    %convert the frequency to rad/sec
    omega=frequency_range(ff)*2*pi;     

    % Pre-calcuate this term before doing the integration
    incident_Im_In = Im_part_incident(m_index , theta_i, phi_i ,    ...
                                omega,  plate.len_x , fluid.c) .*   ...
                     In_part_incident(n_index , theta_i, phi_i ,    ...
                                omega,  plate.len_y , fluid.c);

    % Calculate the transmitted power
    % See Roussos (1985) Eq (25), (24), (23),
    power_transmitted(ff)=dblquad(@transmitted_intensity,0,pi/2,    ...
            0,2*pi,1e-6,[],m_index,n_index,omega_mn,omega,          ...
            incident_Im_In, plate, fluid);            

    %%%
    % Calculate the sound pressure at the microphone location
    
    %%%
    % The generalised modal forcing pressure is calculated as 
    % [Roussos (1985) Eq. (13)]
    %
    % $$ p_{mn} = 8 P_i \bar{I}_m \bar{I}_n $$
    %
    % Note that at the surface of the panel, the incident acoustic wave is
    % reflected, causing a pressure doubling 
    % [See Roussos (1985) just below Eq. (2), and 
    % Bies and Hansen (2009), Sec. 5.9.2, p221]
    % and hence the the factor out the front is $8 \times$ 
    % instead of $4 \times$.
    p_mn(:,ff)= 8 * pressure_incident * incident_Im_In;

    %%% 
    % The generalised modal participation factors are 
    % [Roussos (1985) Eq. (11)]
    %
    % $$ W_{mn} = \frac{ p_{mn} }{ m_p 
    %           \left[
    %               \omega^{2}_{mn} - \omega^2 + (i C_D \omega / m_p)
    %           \right]     }
    % $$
    %
    % However, we'll change the damping term to a structural loss factor
    % $\eta$    [Ewins (1984) ,p 28, Eq. (2.11)]
    %
    % $$ W_{mn} = \frac{ p_{mn} }{ m_p 
    %           \left[
    %               \omega^{2}_{mn} - \omega^2 + (i \eta \omega_{mn}^{2} )
    %           \right]     }
    % $$
    %
%     W_mn = p_mn(:,ff) / plate.m_p ...
%                 ./ (omega_mn.^2  - ones(size(omega_mn))*omega^2     ...
%                     + 1i*2*plate.strloss*omega_mn.^2               ...
%                     );
    W_mn = p_mn(:,ff) / plate.m_p ...
                ./ (omega_mn.^2  - ones(size(omega_mn))*omega^2     ...
                    + 1i*2*plate.damp_ratio*omega_mn*omega               ...
                    );

	transmitted_Im_In =    ...
        Im_part_transmitted(m_index , mic.loc.theta, mic.loc.phi ,  ...
                                omega,  plate.len_x , fluid.c) .*   ...
            In_part_transmitted(n_index , mic.loc.theta, mic.loc.phi , ...
                                omega,  plate.len_y , fluid.c);
                            
                
    %%% 
    % Calculate the pressure at a remote point
    % Calculate the pressure [see Eq (18) Roussos]
    mic_pressure(ff) = -omega^2     ...
                    * fluid.rho * plate.len_x * plate.len_y       ...
                    / (2 * pi * mic.loc.r)                           ...
                    * exp( 1i*omega     ...
                            * ( -mic.loc.r / fluid.c    ...
                                - sin(mic.loc.theta) / ( 2 * fluid.c) ...
                                * ( plate.len_x * cos(mic.loc.phi)  ...
                                    +plate.len_y * sin(mic.loc.phi) )  ...
                                    )   ...
                           )            ...
                    * sum(    W_mn .* transmitted_Im_In );

    %% Calculate Structural Kinetic Energy
    modalparticipationfactors = W_mn;
    ske(ff)     =   (omega.^2) /2 * modalmass   ...
                    * abs(modalparticipationfactors).'   ...
                    * abs(modalparticipationfactors);
    ske(ff)=abs(ske(ff));

    % Calculate the displacement of the plate at the accelerometer
    accel_displ(ff) = accel_plate_mode_shape.' * W_mn;

    % Calculate second version of intensity using
    % Fuller, p226, Eq. (8.2.6)
%     power_transmitted_fuller(ff)=dblquad(@fuller_intensity,0,pi/2,    ...
%             0,2*pi,1e-8,[],m_index,n_index,omega_mn,omega,          ...
%             W_mn, plate, fluid);            

    power_transmitted_fuller(ff)=integral2(@(theta_t,phi_t) ...
            fuller_intensity(theta_t,phi_t,m_index,n_index,    ...
                        omega_mn,omega, W_mn, plate, fluid) ,    ...
            0,pi/2,    ...
            0,2*pi, ...
            'Method','auto');            
       
    
    
        
end; % end of big freq range for loop ff     

% Calculate the transmission loss for the Roussos method
TL_roussos = 10*log10(  power_incident*             ...
                ones(length(frequency_range),1) ./   ...
                power_transmitted);

            
%% Plot the transmission loss for the 3 methods            
figure;
p1=semilogx(    frequency_range,TL_roussos,         ...
                frequency_range,TL_beranek,':',     ...
                frequency_range,TL_sewell,'--');
axis([30 3e4 0 50]);
set(gca,'fontsize',16);
set(p1,'linewidth',3);
xlabel('Frequency [Hz]');
ylabel('Transmission Loss [dB]');
%title('Transmission Loss - Compare With Roussos Paper Fig. 7');
title('Transmission Loss');
legend('Roussos','Infinite Panel','Sewell',2);
grid




%% Return the data
TL.f        = frequency_range'; % analysis frequency range
TL.roussos  = TL_roussos;       % TL calculated using Roussos method
TL.infinite = TL_beranek;       % TL calculated infinite panel theory
TL.TL_0     = TL_0;             % TL for normal incidence
TL.sewell   = TL_sewell;        % TL calculated using Sewell method
TL.ske      = ske;              % structural kinetic energy
TL.accel_displ = accel_displ;   % displacement of plate at accelerometer
TL.power_transmitted = power_transmitted;
TL.power_transmitted_fuller = power_transmitted_fuller;

TL.mic=mic;
TL.mic.pressure=mic_pressure;

% End of the script.
return;

%% Incident Im
function Im_value=Im_part_incident(m_index , theta, phi ,   ...
                                    omega,  plate_len_x , speed_air)
%%%
% See Equations in Roussos (1985) below Eq. (13)

Im_value=zeros(length(m_index),1);

Im_value = m_index * pi .*   ...
            (   ...
                ones(length(m_index),1) - ((-1).^(m_index)) *   ...
                exp(-1i*sin(theta)*cos(phi)*(omega*plate_len_x/speed_air))...
            ) ./    ...
            (   ...
                (m_index*pi).^2 - ones(length(m_index),1)*  ...
                (sin(theta)*cos(phi)*(omega*plate_len_x/speed_air)).^2 ...
            );

test = abs( (m_index*pi).^2 - ones(length(m_index),1)*  ...
            (sin(theta)*cos(phi)*(omega*plate_len_x/speed_air)).^2  );
row_index=find(test<1e-10);        
Im_value(row_index) = -1i/2*sign(sin(theta)*cos(phi));

return; %end of Im_value=Im_part(k_m, k_m_index ,x_loc, plate_len_x);
%--------------------------------------------------
    
%% Incident In
function In_value=In_part_incident(n_index , theta, phi ,   ...
                                    omega,  plate_len_y , speed_air);
%%%
% See Equations in Roussos (1985) below Eq. (13)

In_value=zeros(length(n_index),1);

In_value = n_index * pi .*   ...
            (   ...
                ones(length(n_index),1) - ((-1).^(n_index)) *   ...
                exp(-1i*sin(theta)*sin(phi)*(omega*plate_len_y/speed_air))...
            ) ./    ...
            (   ...
                (n_index*pi).^2 - ones(length(n_index),1)*  ...
                (sin(theta)*sin(phi)*(omega*plate_len_y/speed_air)).^2 ...
            );

test = abs( (n_index*pi).^2 - ones(length(n_index),1)*  ...
            (sin(theta)*sin(phi)*(omega*plate_len_y/speed_air)).^2  );
%row_index=find(test<1e-10);        
row_index= test<1e-10;        
In_value(row_index) = -1i/2*sign(sin(theta)*sin(phi));

return; %end of In_value
%--------------------------------------------------


%% Transmitted Im
function Im_value=Im_part_transmitted(m_index , theta, phi , ...
                                        omega,  plate_len_x , speed_air);
%%%
% See Equations in Roussos (1985) below Eq. (18)
Im_value=zeros(length(m_index),1);

Im_value = m_index * pi .*   ...
            (   ...
                ones(length(m_index),1) - ((-1).^(m_index)) *   ...
                exp(1i*sin(theta)*cos(phi)*(omega*plate_len_x/speed_air))...
            ) ./    ...
            (   ...
                (m_index*pi).^2 - ones(length(m_index),1)*  ...
                (sin(theta)*cos(phi)*(omega*plate_len_x/speed_air)).^2 ...
            );

test = abs( (m_index*pi).^2 - ones(length(m_index),1)*  ...
        (sin(theta)*cos(phi)*(omega*plate_len_x/speed_air)).^2  );
%row_index=find(test<1e-10);        
row_index= test<1e-10;        
Im_value(row_index) = -1i/2*sign(sin(theta)*cos(phi));

return; %end of Im_value=Im_part(k_m, k_m_index ,x_loc, plate_len_x);
%--------------------------------------------------
    
%% Transmitted In
function In_value=In_part_transmitted(n_index , theta, phi ,    ...
                                        omega,  plate_len_y , speed_air);
%%%
% See Equations in Roussos (1985) below Eq. (18)

In_value=zeros(length(n_index),1);

In_value = n_index * pi .*   ...
            (   ...
                ones(length(n_index),1) - ((-1).^(n_index)) *   ...
                exp(1i*sin(theta)*sin(phi)*(omega*plate_len_y/speed_air))...
            ) ./    ...
            (   ...
                (n_index*pi).^2 - ones(length(n_index),1)*  ...
                (sin(theta)*sin(phi)*(omega*plate_len_y/speed_air)).^2 ...
            );

test = abs( (n_index*pi).^2 - ones(length(n_index),1)*  ...
            (sin(theta)*sin(phi)*(omega*plate_len_y/speed_air)).^2  );
row_index=find(test<1e-10);        
Im_value(row_index) = -1i/2*sign(sin(theta)*sin(phi));

return; %end of In_value=In_part(k_m, k_m_index ,x_loc, plate_len_x);
%--------------------------------------------------


%% Transmitted Intensity
function answer=transmitted_intensity(theta_t,phi_t,m_index,n_index,    ...
                        omega_mn,omega, incident_Im_In, plate, fluid);

answer=zeros(size(theta_t));

for aa=1:length(theta_t),
    % See Roussos (1985) Eq. (24)
% 	Q_mn    =   ...
%             Im_part_transmitted(m_index , theta_t(aa), phi_t ,  ...
%                                 omega,  plate.len_x , fluid.c) .*   ...
%             In_part_transmitted(n_index , theta_t(aa), phi_t ,  ...
%                                 omega,  plate.len_y , fluid.c) .*    ...
%             incident_Im_In  ./  ...
%             (   ...
%                 (omega_mn./(ones(size(omega_mn))*omega)).^2     ...
%                 - ones(size(omega_mn))          ...
%                 + 1i*plate.strloss*omega_mn./(ones(size(omega_mn))*omega)...
%             );

	Q_mn    =   ...
            Im_part_transmitted(m_index , theta_t(aa), phi_t ,  ...
                                omega,  plate.len_x , fluid.c) .*   ...
            In_part_transmitted(n_index , theta_t(aa), phi_t ,  ...
                                omega,  plate.len_y , fluid.c) .*    ...
            incident_Im_In  ./  ...
            (   ...
                (omega_mn./(ones(size(omega_mn))*omega)).^2     ...
                - ones(size(omega_mn))          ...
                + 1i*2*plate.damp_ratio*omega_mn./(ones(size(omega_mn))*omega)...
            );
        
	% Equation 23
	% Note, assumes Pi=1 (incident pressure)
	I_t=    fluid.rho/(2*fluid.c) *     ...
            (4*1*plate.len_x*plate.len_y /  ...
                (pi*plate.density*plate.thick)).^2 * ...
            abs(sum(Q_mn)).^2;
    answer(aa)=I_t*sin(theta_t(aa));    
end;
    
return;
%--------------------------------------------------


%% Transmitted Intensity using Fuller / Weaver Equations
function answer=fuller_intensity(theta_t,phi_t,m_index,n_index,    ...
                        omega_mn,omega, W_mn, plate, fluid);

[rows,cols]=size(theta_t);
answer=zeros(rows,cols);

for aa=1:rows,
    for bb=1:cols,
    % p226 Fuller
    alpha = (omega/fluid.c)*plate.len_x*sin(theta_t(aa,bb))*cos(phi_t(aa,bb));
    beta = (omega/fluid.c)*plate.len_y*sin(theta_t(aa,bb))*sin(phi_t(aa,bb));
    
    % Calculate Eq. (8.2.6) Fuller p226
    % Divide it into terms
    
    % Find the indices where m_index and n_index is even and odd
    m_index_even =  find(mod(m_index,2)==0);
    m_index_odd =  find(mod(m_index,2)==1);
    n_index_even =  find(mod(n_index,2)==0);
    n_index_odd =  find(mod(n_index,2)==1);

    numerator_a=zeros(length(m_index),1);
    numerator_a(m_index_odd,1) = cos(alpha/2)*ones(size(m_index_odd));
    numerator_a(m_index_even,1) = sin(alpha/2)*ones(size(m_index_even));
    
    numerator_b=zeros(length(n_index),1);
    numerator_b(n_index_odd,1) = cos(beta/2)*ones(size(n_index_odd));
    numerator_b(n_index_even,1) = sin(beta/2)*ones(size(m_index_even));
    
    term3 = (   numerator_a .* numerator_b                  ...
                ./              ...
                (   ((alpha./(m_index*pi)).^2 - 1) .*  ...
                    ((beta./ (n_index*pi)).^2 - 1)       ...
                )   ...
            ).^2;

    % Drop the 'r^2' term from the denominator as it cancels
    % out with the r^2 term in integration for power in Eq. (8.2.7)
    term2 = (  (omega/fluid.c) * plate.len_x * plate.len_y      ...
            ./ (pi^3 * m_index .* n_index) ).^2;

    term1 = abs(1i*omega*W_mn).^2;
    
    
	% Equation (8.2.7) p227 Fuller
	% Note, assumes Pi=1 (incident pressure)
	I_t=    (2*fluid.rho*fluid.c) *     ...
            sum(term1.*term2.*term3);
    answer(aa,bb)=I_t*sin(theta_t(aa,bb));    
    end;
end;    

return;
%--------------------------------------------------

